# -*- coding: UTF-8 -*-

__revision__ = '$Id: PluginExportPDF.py 1040 2008-11-15 21:13:49Z mikej06 $'

# Copyright (c) 2005-2007 Vasco Nunes
#
# This program is free software; you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation; either version 2 of the License, or
# (at your option) any later version.
#
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU Library General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with this program; if not, write to the Free Software
# 51 Franklin Street, Fifth Floor, Boston, MA  02110-1301, USA

# You may use and distribute this software under the terms of the
# GNU General Public License, version 2 or later

from reportlab.pdfgen import canvas
from reportlab.lib.pagesizes import letter, A4
from reportlab.lib.units import mm, inch
from reportlab.pdfbase import pdfmetrics
from reportlab.pdfbase.ttfonts import TTFont
from reportlab.rl_config import defaultPageSize
from reportlab.rl_config import defaultEncoding
from reportlab.platypus import Image, SimpleDocTemplate, Paragraph, Spacer
from reportlab.lib.styles import getSampleStyleSheet
from xml.sax import saxutils
import os, gtk
import version
import gutils
import string
import sys
import config
from locale import getdefaultlocale
from sqlalchemy import Select, desc

exec_location = os.path.abspath(os.path.dirname(sys.argv[0]))

plugin_name = "PDF"
plugin_description = _("PDF export plugin")
plugin_author = "Vasco Nunes"
plugin_author_email = "<vasco.m.nunes@gmail.com>"
plugin_version = "0.4"

class ExportPlugin:
    def __init__(self, database, locations, parent_window, debug, **kwargs):
        self.db = database
        self.locations = locations
        self.parent = parent_window
        self.config = kwargs['config']
        self.styles = getSampleStyleSheet()
        self.export_simple_pdf()
        self.fontName = ""

    def export_simple_pdf(self):
        """exports a simple movie list to a pdf file"""
        
        if self.config.get('font', '') != '':
                self.fontName = 'custom_font'
                pdfmetrics.registerFont(TTFont(self.fontName, self.config.get('font', '')))
        else:
                self.fontName = "Helvetica"

        basedir = None
        if not self.config is None:
            basedir = self.config.get('export_dir', None, section='export-pdf')
        if basedir is None:
            filename = gutils.file_chooser(_("Export a PDF"), action=gtk.FILE_CHOOSER_ACTION_SAVE, buttons=(gtk.STOCK_CANCEL,gtk.RESPONSE_CANCEL,gtk.STOCK_SAVE,gtk.RESPONSE_OK),name="griffith_simple_list.pdf")
        else:
            filename = gutils.file_chooser(_("Export a PDF"), action=gtk.FILE_CHOOSER_ACTION_SAVE, buttons=(gtk.STOCK_CANCEL,gtk.RESPONSE_CANCEL,gtk.STOCK_SAVE,gtk.RESPONSE_OK),name="griffith_simple_list.pdf",folder=basedir)
        if filename is not False and filename[0]:
            if not self.config is None and filename[1]:
                self.config.set('export_dir', filename[1], section='export-pdf')
                self.config.save()
            overwrite = None
            pdffilename = filename[0].decode('utf-8')
            if os.path.isfile(pdffilename):
                response = gutils.question(self,_("File exists. Do you want to overwrite it?"),1,self.parent)
                if response==-8:
                    overwrite = True
                else:
                    overwrite = False
                    
            if overwrite == True or overwrite is None:
                # filename encoding
                defaultLang, defaultEnc = getdefaultlocale()
                if defaultEnc is None:
                    defaultEnc = 'UTF-8'
                c = SimpleDocTemplate(pdffilename.encode(defaultEnc))
                # data encoding
                #if defaultEncoding == 'WinAnsiEncoding':
                #    defaultEnc = 'cp1252'
                #else:
                defaultEnc = 'utf-8'
                style = self.styles["Normal"]
                Story = [Spacer(1,2*inch)]
                # define some custom stylesheetfont
                total = self.db.Movie.count()
                p = Paragraph("<font name='" + self.fontName +"' size=\"18\">" + saxutils.escape((_("List of films")).encode('utf-8')) + '</font>', self.styles["Heading1"] )
                Story.append(p)
                Story.append(Paragraph(" ",style))
                p = Paragraph("<font name='" + self.fontName +"' size=\"10\">" + saxutils.escape((_("Total Movies: %s") % str(total)).encode('utf-8'))  + '</font>', self.styles["Heading3"])
                Story.append(p)
                Story.append(Paragraph(" ",style))
                movies = Select(self.db.Movie.c)
                # select sort column
                sort_column_name = self.config.get('sortby', 'number', section='mainlist')
                sort_reverse = self.config.get('sortby_reverse', False, section='mainlist')
                do_grouping = True
                for i in sort_column_name.split(','):
                    if i != 'title' and i != 'o_title':
                        do_grouping = False
                    if self.db.Movie.c.has_key(i):
                        if sort_reverse:
                            movies.order_by_clause.append(desc(self.db.Movie.c[i]))
                        else:
                            movies.order_by_clause.append(self.db.Movie.c[i])
                movies = movies.execute().fetchall()
                first_letter = '0'
                for movie in movies:
                    number = movie.number
                    original_title = str(movie.o_title).encode(defaultEnc)
                    title = str(movie.title).encode(defaultEnc)
                    if movie.director:
                        director = ' - ' + str(movie.director).encode(defaultEnc)
                    else:
                        director = ""
                    # group by first letter
                    if do_grouping and title[0] != first_letter:
                        if title[0] in '0123456789':
                            # Group Numbers
                            if first_letter != '0-9':
                                first_letter = '0-9'
                                paragraph_text = '<font name=' + self.fontName + ' size="15">' + saxutils.escape(first_letter) + '</fonts>'
                                p = Paragraph(paragraph_text.decode(defaultEnc), self.styles['Heading2'])
                                Story.append(p)
                        else:
                            first_letter = title[0]
                            paragraph_text = '<font name=' + self.fontName + ' size="15">' + saxutils.escape(first_letter) + '</fonts>'
                            p = Paragraph(paragraph_text.decode(defaultEnc), self.styles['Heading2'])
                            Story.append(p)
                    # add movie title
                    paragraph_text = '<font name=' + self.fontName + ' size="7">' + \
                        '<b>'+ saxutils.escape(title) + '</b>' + \
                        saxutils.escape(' (' + original_title + '), ' + director + ' | ' + str(number)) + \
                        '</font>'
                    p = Paragraph(paragraph_text.decode(defaultEnc), self.styles['Normal'])
                    Story.append(p)
                    if not movie.genre is None:
                        paragraph_text = '<font name=' + self.fontName + ' size="5">' + \
                        '<b>' + _('Genre') + ': </b>' + saxutils.escape(str(movie.genre).encode(defaultEnc)) + \
                        '</font>'
                        p = Paragraph(paragraph_text.decode(defaultEnc), self.styles['Normal'])
                        Story.append(p)
                    if not movie.cast is None:
                        paragraph_text = '<i><font name=' + self.fontName + ' size="5">' + \
                        '<b>' + _('Cast') + ': </b>' + saxutils.escape('; '.join(str(movie.cast).encode(defaultEnc).split("\n")[0:2])) + \
                            '</font></i>'
                        p = Paragraph(paragraph_text.decode(defaultEnc), self.styles['Normal'])
                        Story.append(p)
                c.build(Story, onFirstPage=self.page_template, onLaterPages=self.page_template)
                gutils.info(self, _('PDF has been created.'), self.parent)
            
    def page_template(self, canvas, doc):
        canvas.saveState()
        canvas.setFont(self.fontName,7)
        canvas.drawCentredString(defaultPageSize[0]/2, 40,_("Page %d") % doc.page)
        canvas.setFont(self.fontName,5)
        canvas.drawCentredString(defaultPageSize[0]/2, 20, (_("Document generated by Griffith v")+
            version.pversion+" - Copyright (C) "+version.pyear+" "+
            version.pauthor+" - " + _("Released Under the GNU/GPL License")).encode('utf-8'))
        canvas.restoreState()



# -*- coding: UTF-8 -*-

__revision__ = '$Id: initialize.py 1038 2008-11-15 20:16:06Z mikej06 $'

# Copyright (c) 2005-2008 Vasco Nunes, Piotr Ożarowski
#
# This program is free software; you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation; either version 2 of the License, or
# (at your option) any later version.
#
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU Library General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with this program; if not, write to the Free Software
# 51 Franklin Street, Fifth Floor, Boston, MA  02110-1301, USA

# You may use and distribute this software under the terms of the
# GNU General Public License, version 2 or later

import sys
import os
import string
import gtk
import gutils
import gobject
import gettext
import platform
import re
import gettext
from locale import getdefaultlocale

try:
	import gtkspell
	spell_support = 1
except:
	spell_support = 0

def locations(self):
	defaultLang, defaultEnc = getdefaultlocale()
	if defaultEnc is None:
		defaultEnc = 'UTF-8'
	locations = {}
	locations['exec'] = os.path.abspath(os.path.dirname(sys.argv[0])) # deprecated
	locations['lib']  = os.path.dirname(__file__)
	
	if os.name == 'nt' or os.name.startswith('win'): # win32, win64
		import winshell
		from win32com.shell import shellcon, shell
		import shutil
		
		mydocs = winshell.my_documents()
		locations['movie_plugins']  = "%s\\lib\\plugins\\movie" % locations['exec']
		locations['export_plugins'] = "%s\\lib\\plugins\\export" % locations['exec']
		locations['images']         = "%s\\images" % locations['exec']
		locations['share']          = locations['images']
		locations['glade']          = "%s\\glade\\" % locations['exec']
		locations['desktop']        = ''
		locations['i18n']           = "%s\\i18n" % locations['exec']
		os.environ['PATH'] += ";lib;"
		
		# griffith dir location should point to 'Application Data'
		# this is changed on 0.9.5+svn so we need to make it backward compatible
		if os.path.exists(os.path.join(mydocs, 'griffith').decode(defaultEnc)):
			shutil.move(os.path.join(mydocs, 'griffith').decode(defaultEnc),os.path.join(shell.SHGetFolderPath(0, shellcon.CSIDL_APPDATA, 0, 0), 'griffith').decode(defaultEnc))
		locations['home']           = os.path.join(shell.SHGetFolderPath(0, shellcon.CSIDL_APPDATA, 0, 0), 'griffith').decode(defaultEnc)
		
		# windows hack for locale setting
		lang = os.getenv('LANG')
		if lang is None:
			if defaultLang:
				lang = defaultLang
		if lang:
			os.environ['LANG'] = lang

	elif os.name == 'posix':
		locations['home']  = os.path.join(os.path.expanduser('~'), '.griffith').decode(defaultEnc)
		locations['share'] = os.path.abspath(os.path.join(locations['lib'], '..'))
		locations['glade'] = os.path.join(locations['share'], 'glade')
		locations['i18n']  = os.path.abspath(os.path.join(locations['share'], '..', 'locale'))
		if not os.path.isdir(locations['i18n']):
			locations['i18n'] = os.path.join(locations['share'], 'i18n')
		#some locations
		locations['movie_plugins']  = os.path.join(locations['lib'], 'plugins', 'movie')
		locations['export_plugins'] = os.path.join(locations['lib'], 'plugins', 'export')
		locations['images']  = os.path.join(locations['share'], 'images')
		locations['desktop'] = os.path.join(os.path.expanduser('~'), 'Desktop').decode(defaultEnc)
	else:
		print 'Operating system not supported'
		sys.exit()
	
	from tempfile import gettempdir
	locations['temp'] = gettempdir()
	
	if self._tmp_home is not None: # see gconsole.check_args
		locations['home'] = self._tmp_home.decode(defaultEnc)
		del self._tmp_home

	try:
		if not os.path.exists(locations['home']):
			self.debug.show('Creating %s' % locations['home'])
			os.makedirs(locations['home'])
		else:
			self.debug.show("Using Griffith directory: %s" % locations['home'])
	except OSError:
		self.debug.show('Unable to create griffith directory.')
		raise
		sys.exit()

	if not os.access(locations['home'], os.W_OK):
		self.debug.show('Cannot write to griffith directory, %s' % locations['home'])
		sys.exit()

	# includes plugins in system path for easier importing
	sys.path.append(locations['lib'])
	sys.path.append(locations['movie_plugins'])
	sys.path.append(locations['export_plugins'])
	
	self.locations = locations
	return locations

def location_posters(locations, config):
	if config.get('posters', None) is not None:
		locations['posters']  = os.path.join(locations['home'], config.get('posters'))
	elif config.get('type', 'sqlite', section='database') == 'sqlite':
		dbname = config.get('name', 'griffith', section='database')
		if dbname != 'griffith':
			config['posters'] = 'posters_sqlite_' + dbname
		else:
			config['posters'] = 'posters'
		locations['posters'] = os.path.join(locations['home'], config.get('posters'))
		config.save()
	else:
		config['posters'] = "posters_%(type)s_%(host)s_%(port)s_%(name)s_%(user)s" % config.toDict('database')
		locations['posters'] = os.path.join(locations['home'], config.get('posters'))
		config.save()
	# check if posters dir exists
	if not os.path.isdir(locations['posters']):
		os.makedirs(locations['posters'])

def gui(self):
	self._ = None
	self.debug.show("running on %s - %s" % (os.name, platform.system()))
	if os.name == 'nt' or os.name.startswith('win'):
		self.windows = True
	else:
		self.windows = False
		
	if platform.system() == 'Darwin':
		self.mac = True
	else:
		self.mac = False
	
	self.griffith_dir = self.locations['home']	# deprecated
	
	if self.windows:
		gtk.rc_parse('%s\\gtkrc' % self.locations['exec'])


	gf = os.path.join(self.locations['glade'], 'griffith.glade')
	from widgets import define_widgets
	define_widgets(self, gtk.glade.XML(gf))

	self.pdf_reader = self.config.get('pdf_reader')

def i18n(self, location):
	gettext.bindtextdomain('griffith', location)
	gettext.textdomain('griffith')
	gtk.glade.bindtextdomain('griffith', location)
	gtk.glade.textdomain('griffith')
	gettext.install('griffith', location, unicode=1)

def toolbar(self):
	"""if toolbar is hide in config lets hide the widget"""
	if not self.config.get('view_toolbar', 'True', section='window'):
		self.widgets['toolbar'].hide()
		self.widgets['menu']['toolbar'].set_active(False)

def treeview(self):
	self.treemodel = gtk.TreeStore(str, gtk.gdk.Pixbuf, str, str, str, str, bool, str, str)
	self.widgets['treeview'].set_model(self.treemodel)
	self.widgets['treeview'].set_headers_visible(True)
	# number column
	renderer=gtk.CellRendererText()
	self.number_column=gtk.TreeViewColumn(_('N.'), renderer, text=0)
	self.number_column.set_resizable(True)
	self.number_column.set_sort_column_id(0)
	self.number_column.set_reorderable(True)
	self.widgets['treeview'].append_column(self.number_column)
	# pic column
	renderer=gtk.CellRendererPixbuf()
	self.image_column=gtk.TreeViewColumn(_('Image'), renderer, pixbuf=1)
	self.image_column.set_resizable(False)
	self.image_column.set_reorderable(True)
	self.widgets['treeview'].append_column(self.image_column)
	# original title column
	renderer=gtk.CellRendererText()
	self.otitle_column=gtk.TreeViewColumn(_('Original Title'), renderer, text=2)
	self.otitle_column.set_resizable(True)
	self.otitle_column.set_sort_column_id(2)
	self.otitle_column.set_reorderable(True)
	self.widgets['treeview'].append_column(self.otitle_column)
	# title column
	renderer=gtk.CellRendererText()
	self.title_column=gtk.TreeViewColumn(_('Title'), renderer, text=3)
	self.title_column.set_resizable(True)
	self.title_column.set_sort_column_id(3)
	self.title_column.set_reorderable(True)
	self.widgets['treeview'].append_column(self.title_column)
	# director column
	renderer=gtk.CellRendererText()
	self.director_column=gtk.TreeViewColumn(_('Director'), renderer, text=4)
	self.director_column.set_sort_column_id(4)
	self.director_column.set_resizable(True)
	self.director_column.set_reorderable(True)
	self.widgets['treeview'].append_column(self.director_column)
	# genre column
	renderer=gtk.CellRendererText()
	self.genre_column=gtk.TreeViewColumn(_('Genre'), renderer, text=5)
	self.genre_column.set_sort_column_id(5)
	self.genre_column.set_resizable(True)
	self.genre_column.set_reorderable(True)
	self.widgets['treeview'].append_column(self.genre_column)
	# seen column
	renderer=gtk.CellRendererToggle()
	self.seen_column=gtk.TreeViewColumn(_('Seen it'), renderer, active=6)
	self.seen_column.set_sort_column_id(6)
	self.seen_column.set_resizable(True)
	self.seen_column.set_reorderable(True)
	self.widgets['treeview'].insert_column(self.seen_column, 1)
	# year column
	renderer=gtk.CellRendererText()
	renderer.set_property('xalign', 0.5)
	self.year_column=gtk.TreeViewColumn(_('Year'), renderer, text=7)
	self.year_column.set_sort_column_id(7)
	self.year_column.set_resizable(True)
	self.year_column.set_alignment(0.5)
	self.year_column.set_reorderable(True)
	self.widgets['treeview'].append_column(self.year_column)
	# runtime column
	renderer=gtk.CellRendererText()
	renderer.set_property('xalign', 1)
	self.runtime_column=gtk.TreeViewColumn(_('Runtime'), renderer, text=8)
	self.runtime_column.set_sort_column_id(8)
	self.runtime_column.set_resizable(True)
	self.runtime_column.set_alignment(1)
	self.runtime_column.set_reorderable(True)
	self.widgets['treeview'].append_column(self.runtime_column)
	# rating column
	renderer=gtk.CellRendererText()
	renderer.set_property('xalign', 0.5)
	self.rating_column=gtk.TreeViewColumn(_('Rating'), renderer, text=9)
	self.rating_column.set_sort_column_id(9)
	self.rating_column.set_resizable(True)
	self.rating_column.set_alignment(0.5)
	self.rating_column.set_reorderable(True)
	self.widgets['treeview'].append_column(self.rating_column)
	# reflect saved column order
	columnorder = self.config.get('columnorder', None, section='mainlist')
	if not columnorder is None:
		currentcol = None
		columnordersplitted = re.split('[ \t]*,[ \t]*', columnorder)
		for col in columnordersplitted:
			if col == 'number':
				self.widgets['treeview'].move_column_after(self.number_column, currentcol)
				currentcol = self.number_column
			elif col == 'image':
				self.widgets['treeview'].move_column_after(self.image_column, currentcol)
				currentcol = self.image_column
			elif col == 'otitle':
				self.widgets['treeview'].move_column_after(self.otitle_column, currentcol)
				currentcol = self.otitle_column
			elif col == 'title':
				self.widgets['treeview'].move_column_after(self.title_column, currentcol)
				currentcol = self.title_column
			elif col == 'director':
				self.widgets['treeview'].move_column_after(self.director_column, currentcol)
				currentcol = self.director_column
			elif col == 'genre':
				self.widgets['treeview'].move_column_after(self.genre_column, currentcol)
				currentcol = self.genre_column
			elif col == 'seen':
				self.widgets['treeview'].move_column_after(self.seen_column, currentcol)
				currentcol = self.seen_column
			elif col == 'year':
				self.widgets['treeview'].move_column_after(self.year_column, currentcol)
				currentcol = self.year_column
			elif col == 'runtime':
				self.widgets['treeview'].move_column_after(self.runtime_column, currentcol)
				currentcol = self.runtime_column
			elif col == 'rating':
				self.widgets['treeview'].move_column_after(self.rating_column, currentcol)
				currentcol = self.rating_column
	# add data to treeview
	self.total = int(self.db.Movie.count())
	self.widgets['treeview'].show()

def loans_treeview(self):
	self.loans_treemodel = gtk.TreeStore(str, str, str) # move to self.widgets
	self.widgets['movie']['loan_history'].set_model(self.loans_treemodel)
	self.widgets['movie']['loan_history'].set_headers_visible(True)
	# loan date
	renderer=gtk.CellRendererText()
	self.date_column=gtk.TreeViewColumn(_('Loan Date'), renderer, text=0)
	self.date_column.set_resizable(True)
	self.widgets['movie']['loan_history'].append_column(self.date_column)
	self.date_column.set_sort_column_id(0)
	# return date
	renderer=gtk.CellRendererText()
	self.return_column=gtk.TreeViewColumn(_('Return Date'), renderer, text=1)
	self.return_column.set_resizable(True)
	self.widgets['movie']['loan_history'].append_column(self.return_column)
	# loan to
	renderer=gtk.CellRendererText()
	self.loaner_column=gtk.TreeViewColumn(_('Loaned To'), renderer, text=2)
	self.loaner_column.set_resizable(True)
	self.widgets['movie']['loan_history'].append_column(self.loaner_column)

def lang_treeview(self):
	treeview = self.widgets['add']['lang_treeview']
	self.lang['model'] = gtk.TreeStore(str, str, str, str, str)
	treeview.set_model(self.lang['model'])
	treeview.set_headers_visible(True)

	model = self.lang['lang'] = gtk.ListStore(int, str)
	for i in self.db.Lang.select():
		model.append([i.lang_id, i.name])
	combo = gtk.CellRendererCombo()
	combo.set_property('model', model)
	combo.set_property('text-column', 1)
	combo.set_property('editable', True)
	combo.set_property('has-entry', False)
	combo.connect('edited', self.on_tv_lang_combo_edited, 0)
	column=gtk.TreeViewColumn(_('Language'), combo, text=0)
	column.set_property('min-width', 80)
	column.set_property('resizable', True)
	column.set_sort_column_id(0)
	treeview.append_column(column)
	
	model = self.lang['type'] = gtk.ListStore(int, str)
	#i = 0
	#for lang_type in self._lang_types:
	#	model.append([i, lang_type])
	#	i += 1
	model.append([0, ''])
	model.append([1, _('lector')])
	model.append([2, _('dubbing')])
	model.append([3, _('subtitles')])
	model.append([4, _("commentary")])
	combo = gtk.CellRendererCombo()
	combo.set_property('model', model)
	combo.set_property('text-column', 1)
	combo.set_property('editable', True)
	combo.set_property('has-entry', False)
	combo.connect('edited', self.on_tv_lang_combo_edited, 1)
	column=gtk.TreeViewColumn(_('Type'), combo, text=1)
	column.set_property('min-width', 80)
	column.set_property('resizable', True)
	column.set_sort_column_id(1)
	treeview.append_column(column)

	model = self.lang['acodec'] = gtk.ListStore(int, str)
	for i in self.db.ACodec.select():
		model.append([i.acodec_id, i.name])
	combo = gtk.CellRendererCombo()
	combo.set_property('model', model)
	combo.set_property('text-column', 1)
	combo.set_property('editable', True)
	combo.set_property('has-entry', False)
	combo.connect('edited', self.on_tv_lang_combo_edited, 2)
	column=gtk.TreeViewColumn(_('Codec'), combo, text=2)
	column.set_property('min-width', 80)
	column.set_property('resizable', True)
	column.set_sort_column_id(2)
	treeview.append_column(column)
	
	model = self.lang['achannel'] = gtk.ListStore(int, str)
	for i in self.db.AChannel.select():
		model.append([i.achannel_id, i.name])
	combo = gtk.CellRendererCombo()
	combo.set_property('model', model)
	combo.set_property('text-column', 1)
	combo.set_property('editable', True)
	combo.set_property('has-entry', False)
	combo.connect('edited', self.on_tv_lang_combo_edited, 3)
	column=gtk.TreeViewColumn(_('Channels'), combo, text=3)
	column.set_property('min-width', 80)
	column.set_property('resizable', True)
	column.set_sort_column_id(3)
	treeview.append_column(column)
	
	model = self.lang['subformat'] = gtk.ListStore(int, str)
	for i in self.db.SubFormat.select():
		model.append([i.subformat_id, i.name])
	combo = gtk.CellRendererCombo()
	combo.set_property('model', model)
	combo.set_property('text-column', 1)
	combo.set_property('editable', True)
	combo.set_property('has-entry', False)
	combo.connect('edited', self.on_tv_lang_combo_edited, 4)
	column=gtk.TreeViewColumn(_('Subtitle format'), combo, text=4)
	column.set_property('min-width', 80)
	column.set_property('resizable', True)
	column.set_sort_column_id(4)
	treeview.append_column(column)
	
	treeview.show_all()

def movie_plugins(self):
	"""
	dinamically finds the movie source information plugins
	and fills the plugins drop down list
	"""
	self.plugins = gutils.read_plugins('PluginMovie', \
		self.locations['movie_plugins'])
	self.plugins.sort()
	self.d_plugin = 0
	mcounter = 0
	default_plugin = self.config.get('default_movie_plugin')
	for p in self.plugins:
		plugin_module = os.path.basename(p).replace('.py','')
		plugin_name = plugin_module.replace('PluginMovie','')
		self.widgets['add']['source'].append_text(plugin_name)
		self.widgets['preferences']['default_plugin'].append_text(plugin_name)
		if plugin_name == default_plugin:
			self.widgets['preferences']['default_plugin'].set_active(mcounter)
			self.d_plugin = mcounter
		mcounter = mcounter + 1
	self.widgets['add']['source'].set_active(self.d_plugin)

def export_plugins(self):
	"""
	dinamically finds the available export plugins
	and fills the export menu entry
	"""
	plugins = gutils.read_plugins('PluginExport', \
		self.locations['export_plugins'])
	plugins.sort()
	for p in plugins:
		plugin_module = os.path.basename(p).replace('.py', '')
		plugin_name = plugin_module.replace('PluginExport', '')
		menu_items = gtk.MenuItem(plugin_name)
		self.widgets['menu']['export'].append(menu_items)
		menu_items.connect('activate', self.on_export_activate, plugin_name)
		menu_items.show()
		
def import_plugins(self):
	"""
	dinamically finds the available import plugins
	and fills the import menu entry
	"""
	
	import plugins.imp, math

	fields_to_import = ( 'number','title', 'o_title', 'director', 'year', 'runtime', 'country',
		'seen', 'rating', 'genre', 'studio', 'plot', 'cast', 'notes', 'classification',
		'site', 'o_site', 'trailer', 'medium_id', 'media_num', 'vcodec_id', 'color', 'cond',
		'layers', 'region', 'collection_id', 'volume_id', 'image')

	# glade
	glade_file = gtk.glade.XML(os.path.join(self.locations['glade'], 'import.glade'))
	get = lambda x: glade_file.get_widget(x)
	
	w = self.widgets['import'] = {
		'window'	: get('dialog_import'),
		'pwindow'	: get('dialog_progress'),
		'pabort'	: get('p_abortbutton'),
		'fcw'		: get('fcw'),
		'plugin'	: get('combo_plugin'),
		'author'	: get('l_author'),
		'email'		: get('l_email'),
		'version'	: get('l_version'),
		'description'	: get('l_description'),
		'box_import_1'	: get('box_import_1'),
		'box_import_2'	: get('box_import_2'),
		'box_import_3'	: get('box_import_3'),
		'progress'	: get('l_progress'),
		'progressbar'	: get('progressbar'),
		'fields'	: {},
	}
	get('cancel_button').connect('clicked', plugins.imp.on_abort_button_clicked, self)
	get('import_button').connect('clicked', plugins.imp.on_import_button_clicked, self)
	w['plugin'].connect('changed', plugins.imp.on_import_plugin_changed, w)
	w['window'].set_transient_for(self.widgets['window'])
	w['pwindow'].set_transient_for(self.widgets['window'])
	
	for name in plugins.imp.__all__:
		w['plugin'].append_text(name)
	w['plugin'].set_active(0)
	
	# fields to import
	j = 0
	k = math.ceil( len(self.field_names) / float(3) )
	for i in fields_to_import:
		j = j + 1
		w['fields'][i] = gtk.CheckButton(self.field_names[i])
		w['fields'][i].set_active(True) # TODO: get from config
		if j <= k:
			w['box_import_1'].add(w['fields'][i])
		elif j<= 2*k:
			w['box_import_2'].add(w['fields'][i])
		else:
			w['box_import_3'].add(w['fields'][i])
	w['box_import_1'].show_all()
	w['box_import_2'].show_all()
	w['box_import_3'].show_all()

def people_treeview(self, create=True):
	row = None
	self.p_treemodel = gtk.TreeStore(gobject.TYPE_STRING, gobject.TYPE_STRING, gobject.TYPE_STRING, gobject.TYPE_STRING)
	self.widgets['preferences']['treeview'].set_model(self.p_treemodel)
	self.widgets['preferences']['treeview'].set_headers_visible(True)

	if create==True:
		# name column
		renderer=gtk.CellRendererText()
		column=gtk.TreeViewColumn(_('Name'), renderer, text=0)
		column.set_resizable(True)
		column.set_sort_column_id(0)
		self.widgets['preferences']['treeview'].append_column(column)
		# email column
		renderer=gtk.CellRendererText()
		column=gtk.TreeViewColumn(_('E-mail'),renderer, text=1)
		column.set_resizable(True)
		column.set_sort_column_id(1)
		self.widgets['preferences']['treeview'].append_column(column)
	# add data to treeview
	self.p_treemodel.clear()
	for person in self.db.Person.select(order_by='name ASC'):
		myiter = self.p_treemodel.insert_before(None, None)
		self.p_treemodel.set_value(myiter, 0, str(person.name))
		self.p_treemodel.set_value(myiter, 1, str(person.email))
	self.widgets['preferences']['treeview'].show()

def combos(self):
	i = 0
	for cond in self._conditions:
		self.widgets['preferences']['condition'].insert_text(i, cond)
		self.widgets['add']['condition'].insert_text(i, cond)
		i += 1
	i = 0
	for color in self._colors:
		self.widgets['preferences']['color'].insert_text(i, color)
		self.widgets['add']['color'].insert_text(i, color)
		i += 1
	i = 0
	for region in self._regions:
		self.widgets['preferences']['region'].insert_text(i, region)
		self.widgets['add']['region'].insert_text(i, region)
		i += 1
	i = 0
	for layer in self._layers:
		self.widgets['preferences']['layers'].insert_text(i, layer)
		self.widgets['add']['layers'].insert_text(i, layer)
		i += 1
	i = 0
	pos_to_activate = 0
	selected_criteria = self.config.get('criteria', None, section='mainlist')
	for criteria in self.search_criteria:
		new_criteria = self.field_names[criteria]
		self.widgets['filter']['criteria'].insert_text(i, new_criteria)
		if selected_criteria == new_criteria:
			pos_to_activate = i
		i += 1
	self.widgets['filter']['criteria'].set_active(pos_to_activate)
	i = 0
	for field in self.sort_criteria:
		if field != 'movie_id':
			self.widgets['preferences']['sortby'].insert_text(i, self.field_names[field])
		else:
			self.widgets['preferences']['sortby'].insert_text(i, _('Last added'))
		i += 1
	self.widgets['preferences']['sortby'].set_wrap_width(3)
	self.widgets['preferences']['sortby'].set_active(0) # Number

def dictionaries(self):
	"""initializes data filled dynamically by users"""
	import update
	self.am_tags = {} # dictionary for tag CheckBoxes
	update.update_volume_combo_ids(self)
	update.update_collection_combo_ids(self)
	update.update_loanedto_combo_ids(self)
	update.update_bytag_combo_ids(self)
	fill_volumes_combo(self)
	fill_collections_combo(self)
	fill_loanedto_combo(self)
	fill_bytag_combo(self)
	fill_preferences_tags_combo(self)
	language_combos(self)
	acodec_combos(self)
	achannel_combos(self)
	subformat_combos(self)
	vcodec_combos(self)
	media_combos(self)
	create_tag_vbox(self, widget=self.widgets['add']['tag_vbox'], tab=self.am_tags)
	self.sort_criteria = [ # "[]" because of index() 
		'number', 'o_title', 'title', 'director', 'year', 'runtime', 'country',
		'genre', 'studio', 'media_num', 'rating', 'classification', 'collection_id',
		'volume_id', 'cond', 'layers', 'region', 'movie_id']
	self.search_criteria = (
		'o_title', 'title', 'number', 'director', 'plot', 'cast', 'notes', 'year',
		'runtime', 'country', 'genre', 'studio', 'media_num', 'rating')
	self.field_names = {
		'cast'           : _('Cast'),
		'classification' : _('Classification'),
		'collection_id'  : _('Collection'),
		'color'          : _('Color'),
		'cond'           : _('Condition'),
		'country'        : _('Country'),
		'director'       : _('Director'),
		'genre'          : _('Genre'),
		'image'          : _('Image'),
		'layers'         : _('Layers'),
		'loaned'         : _('Loaned'),
		'media_num'      : _('Discs'),
		'medium_id'      : _('Medium'),
		'notes'          : _('Notes'),
		'number'         : _('Number'),
		'o_site'         : _('Official site'),
		'o_title'        : _('Original Title'),
		'plot'           : _('Plot'),
		'rating'         : _('Rating'),
		'region'         : _('Region'),
		'runtime'        : _('Runtime'),
		'seen'           : _('Seen it'),
		'site'           : _('Site'),
		'studio'         : _('Studio'),
		'title'          : _('Title'),
		'trailer'        : _('Trailer'),
		'vcodec_id'      : _('Video codec'),
		'volume_id'      : _('Volume'),
		'year'           : _('Year')}
	self._conditions = (_('N/A'), _('Damaged'), _('Poor'),  _('Fair'), _('Good'), _('Excellent'))
	self._colors = (_('N/A'), _('Color'), _('Black and White'), _('Mixed'))
	self._lang_types = ('', _('lector'), _('dubbing'), _('subtitles'), _('commentary'))
	self._layers = (_('N/A'), _('Single Side, Single Layer'), _('Single Side, Dual Layer'), _('Dual Side, Single Layer'), _('Dual Side, Dual Layer'))
	self._regions = (
		_('Region 0 (No Region Coding)'),
		_('Region 1 (United States of America, Canada)'),
		_('Region 2 (Europe,including France, Greece, Turkey, Egypt, Arabia, Japan and South Africa)'),
		_('Region 3 (Korea, Thailand, Vietnam, Borneo and Indonesia)'),
		_('Region 4 (Australia and New Zealand, Mexico, the Caribbean, and South America)'),
		_('Region 5 (India, Africa, Russia and former USSR countries)'),
		_('Region 6 (Popular Republic of China)'),
		_('Region 7 (Reserved for Unspecified Special Use)'),
		_('Region 8 (Airlines/Cruise Ships)'),
	)

def web_results(self):
	self.treemodel_results = gtk.TreeStore(str, str)
	self.widgets['results']['treeview'].set_model(self.treemodel_results)
	self.widgets['results']['treeview'].set_headers_visible(False)
	# column ids
	renderer=gtk.CellRendererText()
	self.column1=gtk.TreeViewColumn(None, renderer, text=0)
	self.column1.set_visible(False)
	self.widgets['results']['treeview'].append_column(self.column1)
	# column titles
	renderer=gtk.CellRendererText()
	self.column2=gtk.TreeViewColumn(None, renderer, text=1)
	self.column2.set_resizable(True)
	self.column2.set_sort_column_id(1)
	self.widgets['results']['treeview'].append_column(self.column2)

def gtkspell(self):
	global spell_support
	spell_error = False
	if self.posix and spell_support:
		if self.config.get('gtkspell', False, section='spell') == True:
			if self.config.get('notes', True, section='spell') == True and self.config.get('lang', section='spell') != '':
				try:
					self.notes_spell = gtkspell.Spell(self.widgets['add']['cast'], self.config.get('lang', section='spell'))
				except:
					spell_error = True
			if self.config.get('plot', True, section='spell')==True and self.config.get('lang', section='spell') != '':
				try:
					self.plot_spell = gtkspell.Spell(self.widgets['add']['plot'], self.config.get('lang', section='spell'))
				except:
					spell_error = True
			if spell_error:
				self.debug.show('Dictionary not available. Spellcheck will be disabled.')
				if not self.config.get('notified', False, section='spell'):
					gutils.info(self, _("Dictionary not available. Spellcheck will be disabled. \n" + \
						"Please install the aspell-%s package or adjust the spellchekcer preferences.")%self.config.get('lang', section='spell'), \
						self.widgets['preferences']['window'])
					self.config.set('notified', True, section='spell')
					self.config.save()
	else:
		self.debug.show('Spellchecker is not available')

def preferences(self):
	self.widgets['preferences']['db_type'].insert_text(0,'SQLite3 (internal)')
	self.widgets['preferences']['db_type'].insert_text(1,'PostgreSQL')
	self.widgets['preferences']['db_type'].insert_text(2,'MySQL')
	self.widgets['preferences']['db_type'].insert_text(3,'Microsoft SQL')
	self.widgets['preferences']['db_host'].set_text(self.config.get('host', '', section='database'))
	self.widgets['preferences']['db_port'].set_value(int(self.config.get('port', 0, section='database')))
	self.widgets['preferences']['db_user'].set_text(self.config.get('user', '', section='database'))
	self.widgets['preferences']['db_passwd'].set_text(self.config.get('passwd', '', section='database'))
	self.widgets['preferences']['db_name'].set_text(self.config.get('name', '', section='database'))
	db_type = self.config.get('type', 'sqlite', section='database')
	if db_type == 'postgres':
		self.widgets['preferences']['db_type'].set_active(1)
	elif db_type == 'mysql':
		self.widgets['preferences']['db_type'].set_active(2)
	elif db_type == 'mssql':
		self.widgets['preferences']['db_type'].set_active(3)
	else:
		self.widgets['preferences']['db_type'].set_active(0)

def fill_volumes_combo(self, default=0):
	_tmp = self.initialized
	self.initialized = False # don't refresh main treeview
	self.widgets['add']['volume'].get_model().clear()
	self.widgets['filter']['volume'].get_model().clear()
	for i in self.volume_combo_ids:
		vol_id = self.volume_combo_ids[i]
		if vol_id>0:
			name = self.db.Volume.get_by(volume_id=vol_id).name
		else:
			name = ''
		self.widgets['add']['volume'].insert_text(int(i), str(name))
		# add some white spaces to prevent scrollbar hides parts of the names	
		self.widgets['filter']['volume'].insert_text(int(i), str(name) + '   ')
	self.initialized = _tmp
	self.widgets['add']['volume'].show_all()
	self.widgets['filter']['volume'].show_all()
	self.widgets['filter']['volume'].set_active(0)
	i = gutils.findKey(default, self.volume_combo_ids)
	if i is not None:
		self.widgets['add']['volume'].set_active(int(i))
	self.widgets['add']['volume'].set_wrap_width(3)

def fill_collections_combo(self, default=0):
	_tmp = self.initialized
	self.initialized = False # don't refresh main treeview
	self.widgets['add']['collection'].get_model().clear()
	self.widgets['filter']['collection'].get_model().clear()
	for i in self.collection_combo_ids:
		col_id = self.collection_combo_ids[i]
		if col_id>0:
			name = self.db.Collection.get_by(collection_id=col_id).name
		else:
			name = ''
		self.widgets['add']['collection'].insert_text(int(i), str(name))
		# add some white spaces to prevent scrollbar hides parts of the names	
		self.widgets['filter']['collection'].insert_text(int(i), str(name) + '   ')
	self.initialized = _tmp
	self.initialized = False
	self.widgets['add']['collection'].show_all()
	self.widgets['filter']['collection'].show_all()
	self.widgets['filter']['collection'].set_active(0)
	i = gutils.findKey(default, self.collection_combo_ids)
	if i is not None:
		self.widgets['add']['collection'].set_active(int(i))
	self.widgets['add']['collection'].set_wrap_width(2)

def fill_loanedto_combo(self):
	_tmp = self.initialized
	self.initialized = False # don't refresh main treeview
	self.widgets['filter']['loanedto'].get_model().clear()
	for i in self.loanedto_combo_ids:
		per_id = self.loanedto_combo_ids[i]
		if per_id>0:
			name = self.db.Person.get_by(person_id=per_id).name
		else:
			name = ''
		# add some white spaces to prevent scrollbar hides parts of the names	
		self.widgets['filter']['loanedto'].insert_text(int(i), str(name) + '   ')
	self.initialized = _tmp
	self.widgets['filter']['loanedto'].show_all()
	self.widgets['filter']['loanedto'].set_active(0)

def fill_bytag_combo(self):
	_tmp = self.initialized
	self.initialized = False # don't refresh main treeview
	self.widgets['filter']['tag'].get_model().clear()
	for i in self.bytag_combo_ids:
		id = self.bytag_combo_ids[i]
		if id>0:
			name = self.db.Tag.get_by(tag_id=id).name
		else:
			name = ''
		# add some white spaces to prevent scrollbar hides parts of the names	
		self.widgets['filter']['tag'].insert_text(int(i), str(name) + '   ')
	self.initialized = _tmp
	self.widgets['filter']['tag'].show_all()
	self.widgets['filter']['tag'].set_active(0)

def fill_preferences_tags_combo(self):
	_tmp = self.initialized
	self.initialized = False # don't refresh main treeview
	self.widgets['preferences']['tag_name'].get_model().clear()
	self.tags_ids = {}
	i = 0
	for tag in self.db.Tag.select():
		self.tags_ids[i] = tag.tag_id
		self.widgets['preferences']['tag_name'].insert_text(int(i), str(tag.name))
		i += 1
	self.initialized = _tmp
	self.widgets['preferences']['tag_name'].show_all()

def language_combos(self):
	self.widgets['preferences']['lang_name'].get_model().clear()
	self.languages_ids = {}
	self.languages_ids[0] = 0	# empty one
	self.widgets['preferences']['lang_name'].insert_text(0, '')
	i = 1
	for lang in self.db.Lang.select():
		self.languages_ids[i] = lang.lang_id
		self.widgets['preferences']['lang_name'].insert_text(int(i), str(lang.name))
		i += 1
	self.widgets['preferences']['lang_name'].show_all()
	# add movie languages treeview
	self.lang['lang'].clear()
	for i in self.db.Lang.select():
		self.lang['lang'].append([i.lang_id, i.name])
def acodec_combos(self):
	self.widgets['preferences']['acodec_name'].get_model().clear()
	self.acodecs_ids = {}
	self.acodecs_ids[0] = 0	# empty one
	self.widgets['preferences']['acodec_name'].insert_text(0, '')
	i = 1
	for acodec in self.db.ACodec.select():
		self.acodecs_ids[i] = acodec.acodec_id
		self.widgets['preferences']['acodec_name'].insert_text(int(i), str(acodec.name))
		i += 1
	self.widgets['preferences']['acodec_name'].show_all()
	# add movie languages treeview
	self.lang['acodec'].clear()
	for i in self.db.ACodec.select():
		self.lang['acodec'].append([i.acodec_id, i.name])
def achannel_combos(self):
	self.widgets['preferences']['achannel_name'].get_model().clear()
	self.achannels_ids = {}
	self.achannels_ids[0] = 0	# empty one
	self.widgets['preferences']['achannel_name'].insert_text(0, '')
	i = 1
	for achannel in self.db.AChannel.select():
		self.achannels_ids[i] = achannel.achannel_id
		self.widgets['preferences']['achannel_name'].insert_text(int(i), str(achannel.name))
		i += 1
	self.widgets['preferences']['achannel_name'].show_all()
	# add movie languages treeview
	self.lang['achannel'].clear()
	for i in self.db.AChannel.select():
		self.lang['achannel'].append([i.achannel_id, i.name])
def subformat_combos(self):
	self.widgets['preferences']['subformat_name'].get_model().clear()
	self.subformats_ids = {}
	self.subformats_ids[0] = 0	# empty one
	self.widgets['preferences']['subformat_name'].insert_text(0, '')
	i = 1
	for subformat in self.db.SubFormat.select():
		self.subformats_ids[i] = subformat.subformat_id
		self.widgets['preferences']['subformat_name'].insert_text(int(i), str(subformat.name))
		i += 1
	self.widgets['preferences']['subformat_name'].show_all()
	# add movie languages treeview
	self.lang['subformat'].clear()
	for i in self.db.SubFormat.select():
		self.lang['subformat'].append([i.subformat_id, i.name])

def media_combos(self):
	# clear data
	self.widgets['preferences']['medium_name'].get_model().clear()
	self.widgets['preferences']['media'].get_model().clear()
	self.widgets['add']['media'].get_model().clear()
	
	self.media_ids = {}

	self.media_ids[0] = None
	self.widgets['preferences']['medium_name'].insert_text(0, '')
	self.widgets['add']['media'].insert_text(0, _('N/A'))
	self.widgets['preferences']['media'].insert_text(0, _('N/A'))
	i = 1
	for medium in self.db.Medium.select():
		self.media_ids[i] = medium.medium_id
		self.widgets['preferences']['medium_name'].insert_text(int(i), str(medium.name))
		self.widgets['add']['media'].insert_text(int(i), str(medium.name))
		self.widgets['preferences']['media'].insert_text(int(i), str(medium.name))
		i += 1

	self.widgets['preferences']['medium_name'].show_all()
	self.widgets['add']['media'].show_all()
	self.widgets['preferences']['media'].show_all()
	if self.config.has_key('media', section='defaults'):
		pos = gutils.findKey(self.config.get('media', section='defaults'), self.media_ids)
		if pos  is not None:
			self.widgets['preferences']['media'].set_active(int(pos))
		else:
			self.widgets['preferences']['media'].set_active(0)
	else:
		self.widgets['preferences']['media'].set_active(0)

def vcodec_combos(self):
	# clear data
	self.widgets['preferences']['vcodec_name'].get_model().clear()
	self.widgets['preferences']['vcodec'].get_model().clear()
	self.widgets['add']['vcodec'].get_model().clear()
	
	self.vcodecs_ids = {}
	
	self.vcodecs_ids[0] = None
	self.widgets['preferences']['vcodec_name'].insert_text(0, '')
	self.widgets['add']['vcodec'].insert_text(0, _('N/A'))
	self.widgets['preferences']['vcodec'].insert_text(0, _('N/A'))
	i = 1
	for vcodec in self.db.VCodec.select():
		self.vcodecs_ids[i] = vcodec.vcodec_id
		self.widgets['preferences']['vcodec_name'].insert_text(int(i), str(vcodec.name))
		self.widgets['add']['vcodec'].insert_text(int(i), str(vcodec.name))
		self.widgets['preferences']['vcodec'].insert_text(int(i), str(vcodec.name))
		i += 1

	self.widgets['preferences']['vcodec_name'].show_all()
	self.widgets['add']['vcodec'].show_all()
	self.widgets['preferences']['vcodec'].show_all()
	
	pos = gutils.findKey(self.config.get('vcodec', 0, section='defaults'), self.vcodecs_ids)
	if pos is not None:
		self.widgets['preferences']['vcodec'].set_active(int(pos))
	else:
		self.widgets['preferences']['vcodec'].set_active(0)

def create_tag_vbox(self, widget, tab):
	for i in widget.get_children():
		i.destroy()
	for i in self.tags_ids:
		tag_id = self.tags_ids[i]
		tag_name = self.db.Tag.get_by(tag_id=tag_id).name
		tab[i] = gtk.CheckButton(str(tag_name))
		tab[i].set_active(False)
		widget.pack_start(tab[i])
	widget.show_all()

def remove_hbox(self, widget, tab):
	number = len(widget.get_children())-1	# last box number
	try:
		tab.pop()
		widget.remove(widget.get_children().pop())
	except:
		self.debug.show('List is empty')
	widget.show_all()




# -*- coding: UTF-8 -*-

__revision__ = '$Id: PluginMovieFilmDb.py 1040 2008-11-15 21:13:49Z mikej06 $'

# Copyright (c) 2007 Michael Jahn
#
# This program is free software; you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation; either version 2 of the License, or
# (at your option) any later version.
#
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU Library General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with this program; if not, write to the Free Software
# 51 Franklin Street, Fifth Floor, Boston, MA  02110-1301, USA

# You may use and distribute this software under the terms of the
# GNU General Public License, version 2 or later

import gutils
import movie
import string

plugin_name		= 'FilmDb.de'
plugin_description	= 'FILMDB.DE'
plugin_url		= 'www.filmdb.de'
plugin_language		= _('German')
plugin_author		= 'Michael Jahn'
plugin_author_email	= '<mikej06@hotmail.com>'
plugin_version		= '1.0'

class Plugin(movie.Movie):
	def __init__(self, id):
		self.encode='iso-8859-1'
		self.movie_id = id
		self.url = "http://www.filmdb.de/filmanzeige.php?alle=1&filmid=" + self.movie_id

	def initialize(self):
		self.tmp_page = gutils.trim(self.page, "<h1>Filmdatenbank - ", "Kommentare</a>")
	
	def get_image(self):
		self.image_url = gutils.trim(self.tmp_page, '<td background="', '"');

	def get_o_title(self):
		self.o_title = gutils.trim(self.page, '<h1>Filmdatenbank - ', '</h1>')

	def get_title(self):
		self.title = gutils.trim(self.page, '<h1>Filmdatenbank - ', '</h1>')

	def get_director(self):
		self.director = gutils.after(gutils.trim(self.tmp_page, 'regisseursuche.php', '</a>'), '>')

	def get_plot(self):
		self.plot = gutils.trim(self.tmp_page, '>Inhalt</strong>', '<td width="150" valign="top">')
		self.plot = self.plot.replace('\t', '')
		self.plot = self.plot.replace('\n', '')
		self.plot = self.plot.replace('„', '"')
		self.plot = self.plot.replace('“', '"')

	def get_year(self):
		elements = string.split(self.tmp_page, 'landjahrsuche.php')
		self.year = gutils.trim(elements[2], '>', '</a>') + '\n'

	def get_runtime(self):
		self.runtime = ""
		tmp = gutils.trim(self.tmp_page, '</a>  – ', ' Stunden')
		if tmp <> '':
			elements = string.split(tmp, ':')
			try:
				hours = int(elements[0])
				mins = int(elements[1])
				self.runtime = str(hours * 60 + mins)
			except:
				self.runtime = ""

	def get_genre(self):
		self.genre = gutils.after(gutils.trim(self.tmp_page, 'genresuche.php', '</a>'), '>')

	def get_cast(self):
		self.cast = ""
		elements = string.split(self.tmp_page, 'schauspielersuche.php')
		elements[0] = ''
		for element in elements:
			if element <> '':
				self.cast = self.cast + gutils.trim(element, '>', '</a>') + '\n'

	def get_classification(self):
		self.classification = ""

	def get_studio(self):
		self.studio = ""

	def get_o_site(self):
		self.o_site = ""

	def get_site(self):
		self.site = "http://www.filmdb.de/filmanzeige.php?filmid=" + self.movie_id

	def get_trailer(self):
		self.trailer = ""

	def get_country(self):
		elements = string.split(self.tmp_page, 'landjahrsuche.php')
		self.country = gutils.trim(elements[1], '>', '</a>') + '\n'

	def get_rating(self):
		self.rating = gutils.trim(self.tmp_page, 'Unsere User haben diesen Film mit ', ' bewertet.')
		self.rating = self.rating.replace('%', '')
		self.rating = gutils.strip_tags(self.rating)
		elements = self.rating.split('.')
		try:
			tmprating = int(elements[0])
			self.rating = str(tmprating / 10)
		except:
			self.rating = '0'

	def get_notes(self):
		self.notes = ""

class SearchPlugin(movie.SearchMovie):

	def __init__(self):
		self.original_url_search   = "http://www.filmdb.de/globalsuche.php?name="
		self.translated_url_search = "http://www.filmdb.de/globalsuche.php?name="
		self.encode='iso-8859-1'

	def search(self,parent_window):
		self.open_search(parent_window)
		return gutils.trim(self.page, "<span class=font_normal>", "<table width=590")

	def get_searches(self):
		elements = string.split(self.page, "<!--")
		elements[0] = ''
		for element in elements:
			if element <> '':
				self.ids.append(gutils.trim(element, "filmid=", ">"))
				self.titles.append(gutils.trim(
					gutils.after(element, "filmid="), ">", "<") + " - " +
					gutils.trim(gutils.after(element, "</a>"), "<td>", "</td>") + " - " +
					gutils.trim(gutils.after(gutils.after(element, "<td>"), "<td>"), "<td>", "</td>"))



# -*- coding: UTF-8 -*-

__revision__ = '$Id: people.py 1040 2008-11-15 21:13:49Z mikej06 $'

# Copyright (c) 2005-2008 Vasco Nunes, Piotr Ożarowski
#
# This program is free software; you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation; either version 2 of the License, or
# (at your option) any later version.
#
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU Library General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with this program; if not, write to the Free Software
# 51 Franklin Street, Fifth Floor, Boston, MA  02110-1301, USA

# You may use and distribute this software under the terms of the
# GNU General Public License, version 2 or later

import gutils

def show_people_window(self):
	self.widgets['people']['window'].show()

def hide_people_window(self):
	self.widgets['people']['window'].hide()

def add_person(self):
	clear_person(self)
	self.widgets['person']['window'].show()

def add_person_cancel(self):
	self.widgets['person']['window'].hide()
	self.widgets['people']['window'].present()

def clear_person(self):
	self.widgets['person']['name'].set_text("")
	self.widgets['person']['email'].set_text("")
	self.widgets['person']['phone'].set_text("")

def add_person_db(self):
	if self.widgets['person']['name'].get_text().decode('utf-8'):
		p = self.db.Person()
		p.name = self.widgets['person']['name'].get_text().decode('utf-8')
		p.email = self.widgets['person']['email'].get_text().decode('utf-8')
		p.phone = self.widgets['person']['phone'].get_text().decode('utf-8')
		self.widgets['person']['window'].hide()
		if p.add_to_db():
			myiter = self.p_treemodel.insert_after(None, None)
			self.p_treemodel.set_value(myiter,0,str(self.widgets['person']['name'].get_text().decode('utf-8')))
			self.p_treemodel.set_value(myiter,1,str(self.widgets['person']['email'].get_text().decode('utf-8')))
		self.widgets['people']['window'].present()
	else:
		gutils.error(self.widgets['results']['window'],_("You should fill the person name"))

def edit_person(self):
	try:
		treeselection = self.widgets['people']['treeview'].get_selection()
		(tmp_model, tmp_iter) = treeselection.get_selected()
		name = tmp_model.get_value(tmp_iter,0).decode('utf-8')
	except:
		return
	p = self.db.Person.get_by(name=name)
	if p is not None:
		self.widgets['person']['e_name'].set_text(str(p.name))
		self.widgets['person']['e_email'].set_text(str(p.email))
		self.widgets['person']['e_phone'].set_text(str(p.phone))
		self.widgets['person']['e_id'].set_text(str(p.person_id))
	self.widgets['person']['e_window'].show()

def edit_person_cancel(self):
	self.widgets['person']['e_window'].hide()
	self.widgets['people']['window'].present()

def update_person(self):
	p = self.db.Person.get_by(person_id=self.widgets['person']['e_id'].get_text().decode('utf-8'))
	if p is None:
		return False
	p.name = self.widgets['person']['e_name'].get_text().decode('utf-8')
	p.email = self.widgets['person']['e_email'].get_text().decode('utf-8')
	p.phone = self.widgets['person']['e_phone'].get_text().decode('utf-8')
	if p.update_in_db():
		self.update_statusbar(_("Record updated"))
		edit_person_cancel(self)
		self.p_treemodel.clear()
		for p in self.db.Person.select(order_by='name ASC'):
			myiter = self.p_treemodel.insert_before(None, None)
			self.p_treemodel.set_value(myiter, 0, str(p.name))
			self.p_treemodel.set_value(myiter, 1, str(p.email))

def delete_person(self):
	response = None
	has_history = False
	has_history_msg = ''
	try:
		treeselection = self.widgets['people']['treeview'].get_selection()
		(tmp_model, tmp_iter) = treeselection.get_selected()
		person = tmp_model.get_value(tmp_iter,0).decode('utf-8')
	except:
		return
	person = self.db.Person.get_by(name=person)
	if not person:
		return False
	data = self.db.Loan.select_by(person_id=person.person_id, return_date=None)
	if len(data)>0:
		gutils.info(self, _("This person has loaned films from you. Return them first."), self.widgets['people']['window'])
		return False
	data = self.db.Loan.select_by(person_id=person.person_id)
	if len(data)>0:
		has_history = True
		has_history_msg = _("This person has data in the loan history. This data will be erased if you continue.")
	response = gutils.question(self,_("%s\nAre you sure you want to delete this person?" % has_history_msg), \
		1, self.widgets['people']['window'])

	if response == -8:
		treeselection = self.widgets['people']['treeview'].get_selection()
		(tmp_model, tmp_iter) = treeselection.get_selected()
		if person.remove_from_db():
			self.p_treemodel.remove(tmp_iter)
			self.treeview_clicked()
	self.widgets['people']['window'].present()




# -*- coding: UTF-8 -*-

__revision__ = '$Id: PluginMoviePTGate.py 1040 2008-11-15 21:13:49Z mikej06 $'

# Copyright (c) 2005-2007 Vasco Nunes, Piotr Ozarowski
#
# This program is free software; you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation; either version 2 of the License, or
# (at your option) any later version.
#
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU Library General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with this program; if not, write to the Free Software
# 51 Franklin Street, Fifth Floor, Boston, MA  02110-1301, USA

# You may use and distribute this software under the terms of the
# GNU General Public License, version 2 or later

import gutils
import movie
import string

plugin_name = "PTGate"
plugin_description = "Cinema PTGate"
plugin_url = "www.cinema.ptgate.pt"
plugin_language = _("Portuguese")
plugin_author = "Vasco Nunes"
plugin_author_email="<vasco.m.nunes@gmail.com>"
plugin_version = "0.4.1"

class Plugin(movie.Movie):
	def __init__(self, id):
		self.encode='iso-8859-1'
		self.movie_id = id
		self.url = "http://cinema.ptgate.pt/filme.php?code=" + str(self.movie_id)

	def get_image(self):
		self.image_url = "http://cinema.ptgate.pt/Movies/" + str(self.movie_id)+".jpg"

	def get_o_title(self):
		self.o_title = string.capwords(gutils.trim(self.page,"<b class=title>","</b><br>") )

	def get_title(self):
		self.title = string.capwords(gutils.trim(self.page,"class=subtitle>","</b><br>") )

	def get_director(self):
		self.director = gutils.trim(self.page,"<b>realização</b><br>","<br><br><b>")

	def get_plot(self):
		self.plot = gutils.trim(self.page,"<b>sinopse</b><br>","<br><br>")
		self.plot = string.replace(self.plot,"'","\"")
		self.plot = string.replace(self.plot,"'","\"")

	def get_year(self):
		self.year = gutils.trim(self.page,"<br><b>ano</b><br>","<br><br><b>pa")

	def get_runtime(self):
		self.runtime = ""

	def get_genre(self):
		self.genre = gutils.trim(self.page,"nero</b><br>","<br><br><b>realiza")
		self.genre = string.replace(self.genre," | ",", ")

	def get_cast(self):
		self.cast = ""
		self.cast = gutils.trim(self.page,"pretes</b><br>","<br><br><b>data de estreia</b><br>")
		self.cast = string.replace(self.cast,"<br> ","\n")
		self.cast = string.replace(self.cast,"<br>", "\n")
		self.cast = gutils.strip_tags(self.cast)

	def get_classification(self):
		self.classification = ""

	def get_studio(self):
		self.studio = ""

	def get_o_site(self):
		self.o_site = gutils.trim(self.page,"tio oficial</b><br><a class=external href='", "'>")

	def get_site(self):
		self.site = gutils.trim(self.page,"<br><b>imdb</b><br><a class=external href='", "'>www.imdb.com")

	def get_trailer(self):
		self.trailer = gutils.trim(self.page,"trailer</b><br><a class=external href='", "'>visionar")

	def get_country(self):
		self.country = gutils.trim(self.page,"s</b><br>","<br><br><b>g")
		
	def get_notes(self):
		self.notes = gutils.trim(self.page,"prémios e nomeações</b><br><b>", "<td width=6 background='Images/group_r.gif'")
		self.notes = gutils.strip_tags(self.notes)

	def get_rating(self):
		tmp=gutils.trim(self.page,"'Images/mstar",".gif")
		if(tmp):
			self.rating = float(tmp)
			if (self.rating>10):
				self.rating = int(self.rating/10)
				self.rating = self.rating*2
		else:
			self.rating=""

class SearchPlugin(movie.SearchMovie):
	def __init__(self):
		self.original_url_search	= "http://cinema.ptgate.pt/pesquisa-filme.php?titulo="
		self.translated_url_search	= "http://cinema.ptgate.pt/pesquisa-filme.php?titulo="
		self.encode='iso-8859-1'

	def search(self,parent_window):
		self.open_search(parent_window)
		self.sub_search()
		return self.page

	def sub_search(self):
		self.page = gutils.trim(self.page,"<!-- DISPLAY RESULTS -->", "<!-- /DISPLAY RESULTS -->")

	def get_searches(self):
		elements = string.split(self.page,"</td><td>")
		self.number_results = elements[-1]

		if (len(elements[0])):
			for element in elements:
				element = string.replace(element,"</a>","")
				self.ids.append(gutils.trim(element,"?code=","'>"))
				self.titles.append(gutils.convert_entities(gutils.trim(element,"'>","</td><td width=6")))
		else:
			self.number_results = 0



# objectstore.py
# Copyright (C) 2005, 2006, 2007 Michael Bayer mike_mp@zzzcomputing.com
#
# This module is part of SQLAlchemy and is released under
# the MIT License: http://www.opensource.org/licenses/mit-license.php

from sqlalchemy import util, exceptions, sql, engine
from sqlalchemy.orm import unitofwork, query
from sqlalchemy.orm.mapper import object_mapper as _object_mapper
from sqlalchemy.orm.mapper import class_mapper as _class_mapper
from sqlalchemy.orm import util as mapperutil
import weakref
import sqlalchemy

class SessionTransaction(object):
    """Represents a Session-level Transaction.

    This corresponds to one or more sqlalchemy.engine.Transaction
    instances behind the scenes, with one Transaction per Engine in
    use.

    The SessionTransaction object is **not** threadsafe.
    """

    def __init__(self, session, parent=None, autoflush=True):
        self.session = session
        self.connections = {}
        self.parent = parent
        self.autoflush = autoflush

    def connection(self, mapper_or_class, entity_name=None):
        if isinstance(mapper_or_class, type):
            mapper_or_class = _class_mapper(mapper_or_class, entity_name=entity_name)
        engine = self.session.get_bind(mapper_or_class)
        return self.get_or_add(engine)

    def _begin(self):
        return SessionTransaction(self.session, self)

    def add(self, bind):
        if self.parent is not None:
            return self.parent.add(bind)

        if self.connections.has_key(bind.engine):
            raise exceptions.InvalidRequestError("Session already has a Connection associated for the given %sEngine" % (isinstance(bind, engine.Connection) and "Connection's " or ""))
            
        return self.get_or_add(bind)

    def get_or_add(self, bind):
        if self.parent is not None:
            return self.parent.get_or_add(bind)
            
        if self.connections.has_key(bind):
            return self.connections[bind][0]

        if not isinstance(bind, engine.Connection):
            e = bind
            c = bind.contextual_connect()
        else:
            e = bind.engine
            c = bind
            if e in self.connections:
                raise exceptions.InvalidRequestError("Session already has a Connection associated for the given Connection's Engine")

        self.connections[bind] = self.connections[e] = (c, c.begin(), c is not bind)
        return self.connections[bind][0]

    def commit(self):
        if self.parent is not None:
            return
        if self.autoflush:
            self.session.flush()
        for t in util.Set(self.connections.values()):
            t[1].commit()
        self.close()

    def rollback(self):
        if self.parent is not None:
            self.parent.rollback()
            return
        for k, t in self.connections.iteritems():
            t[1].rollback()
        self.close()

    def close(self):
        if self.parent is not None:
            return
        for t in self.connections.values():
            if t[2]:
                t[0].close()
        self.session.transaction = None

    def __enter__(self):
        return self

    def __exit__(self, type, value, traceback):
        if self.session.transaction is None:
            return
        if type is None:
            self.commit()
        else:
            self.rollback()

class Session(object):
    """Encapsulates a set of objects being operated upon within an
    object-relational operation.

    The Session object is **not** threadsafe.  For thread-management
    of Sessions, see the ``sqlalchemy.ext.sessioncontext`` module.
    """

    def __init__(self, bind=None, bind_to=None, hash_key=None, import_session=None, echo_uow=False, weak_identity_map=False):
        if import_session is not None:
            self.uow = unitofwork.UnitOfWork(identity_map=import_session.uow.identity_map, weak_identity_map=weak_identity_map)
        else:
            self.uow = unitofwork.UnitOfWork(weak_identity_map=weak_identity_map)

        self.bind = bind or bind_to
        self.binds = {}
        self.echo_uow = echo_uow
        self.weak_identity_map = weak_identity_map
        self.transaction = None
        if hash_key is None:
            self.hash_key = id(self)
        else:
            self.hash_key = hash_key
        _sessions[self.hash_key] = self

    def _get_echo_uow(self):
        return self.uow.echo

    def _set_echo_uow(self, value):
        self.uow.echo = value
    echo_uow = property(_get_echo_uow,_set_echo_uow)
    
    bind_to = property(lambda self:self.bind)

    def create_transaction(self, **kwargs):
        """Return a new ``SessionTransaction`` corresponding to an
        existing or new transaction.

        If the transaction is new, the returned ``SessionTransaction``
        will have commit control over the underlying transaction, else
        will have rollback control only.
        """

        if self.transaction is not None:
            return self.transaction._begin()
        else:
            self.transaction = SessionTransaction(self, **kwargs)
            return self.transaction

    def connect(self, mapper=None, **kwargs):
        """Return a unique connection corresponding to the given mapper.

        This connection will not be part of any pre-existing
        transactional context.
        """

        return self.get_bind(mapper).connect(**kwargs)

    def connection(self, mapper, **kwargs):
        """Return a ``Connection`` corresponding to the given mapper.

        Used by the ``execute()`` method which performs select
        operations for ``Mapper`` and ``Query``.

        If this ``Session`` is transactional, the connection will be in
        the context of this session's transaction.  Otherwise, the
        connection is returned by the ``contextual_connect()`` method, which
        some Engines override to return a thread-local connection, and
        will have `close_with_result` set to `True`.

        The given `**kwargs` will be sent to the engine's
        ``contextual_connect()`` method, if no transaction is in
        progress.
        """

        if self.transaction is not None:
            return self.transaction.connection(mapper)
        else:
            return self.get_bind(mapper).contextual_connect(**kwargs)

    def execute(self, mapper, clause, params, **kwargs):
        """Using the given mapper to identify the appropriate ``Engine``
        or ``Connection`` to be used for statement execution, execute the
        given ``ClauseElement`` using the provided parameter dictionary.

        Return a ``ResultProxy`` corresponding to the execution's results.

        If this method allocates a new ``Connection`` for the operation,
        then the ``ResultProxy`` 's ``close()`` method will release the
        resources of the underlying ``Connection``, otherwise its a no-op.
        """
        return self.connection(mapper, close_with_result=True).execute(clause, params, **kwargs)

    def scalar(self, mapper, clause, params, **kwargs):
        """Like execute() but return a scalar result."""

        return self.connection(mapper, close_with_result=True).scalar(clause, params, **kwargs)

    def close(self):
        """Close this Session."""

        self.clear()
        if self.transaction is not None:
            self.transaction.close()

    def clear(self):
        """Remove all object instances from this ``Session``.

        This is equivalent to calling ``expunge()`` for all objects in
        this ``Session``.
        """

        for instance in self:
            self._unattach(instance)
        echo = self.uow.echo
        self.uow = unitofwork.UnitOfWork(weak_identity_map=self.weak_identity_map)
        self.uow.echo = echo

    def mapper(self, class_, entity_name=None):
        """Given a ``Class``, return the primary ``Mapper`` responsible for
        persisting it."""

        return _class_mapper(class_, entity_name = entity_name)

    def bind_mapper(self, mapper, bind):
        """Bind the given `mapper` to the given ``Engine`` or ``Connection``.

        All subsequent operations involving this ``Mapper`` will use the
        given `bind`.
        """

        self.binds[mapper] = bind

    def bind_table(self, table, bind):
        """Bind the given `table` to the given ``Engine`` or ``Connection``.

        All subsequent operations involving this ``Table`` will use the
        given `bind`.
        """

        self.binds[table] = bind

    def get_bind(self, mapper):
        """Return the ``Engine`` or ``Connection`` which is used to execute
        statements on behalf of the given `mapper`.

        Calling ``connect()`` on the return result will always result
        in a ``Connection`` object.  This method disregards any
        ``SessionTransaction`` that may be in progress.

        The order of searching is as follows:

        1. if an ``Engine`` or ``Connection`` was bound to this ``Mapper``
           specifically within this ``Session``, return that ``Engine`` or
           ``Connection``.

        2. if an ``Engine`` or ``Connection`` was bound to this `mapper` 's
           underlying ``Table`` within this ``Session`` (i.e. not to the ``Table``
           directly), return that ``Engine`` or ``Connection``.

        3. if an ``Engine`` or ``Connection`` was bound to this ``Session``,
           return that ``Engine`` or ``Connection``.

        4. finally, return the ``Engine`` which was bound directly to the
           ``Table`` 's ``MetaData`` object.

        If no ``Engine`` is bound to the ``Table``, an exception is raised.
        """

        if mapper is None:
            return self.bind
        elif self.binds.has_key(mapper):
            return self.binds[mapper]
        elif self.binds.has_key(mapper.mapped_table):
            return self.binds[mapper.mapped_table]
        elif self.bind is not None:
            return self.bind
        else:
            e = mapper.mapped_table.engine
            if e is None:
                raise exceptions.InvalidRequestError("Could not locate any Engine or Connection bound to mapper '%s'" % str(mapper))
            return e

    def query(self, mapper_or_class, *addtl_entities, **kwargs):
        """Return a new ``Query`` object corresponding to this ``Session`` and
        the mapper, or the classes' primary mapper.
        """
        
        entity_name = kwargs.pop('entity_name', None)
        
        if isinstance(mapper_or_class, type):
            q = query.Query(_class_mapper(mapper_or_class, entity_name=entity_name), self, **kwargs)
        else:
            q = query.Query(mapper_or_class, self, **kwargs)
            
        for ent in addtl_entities:
            q = q.add_entity(ent)
        return q

    def _sql(self):
        class SQLProxy(object):
            def __getattr__(self, key):
                def call(*args, **kwargs):
                    kwargs[engine] = self.engine
                    return getattr(sql, key)(*args, **kwargs)

    sql = property(_sql)

    def flush(self, objects=None):
        """Flush all the object modifications present in this session
        to the database.

        `objects` is a list or tuple of objects specifically to be
        flushed; if ``None``, all new and modified objects are flushed.
        """

        self.uow.flush(self, objects)

    def get(self, class_, ident, **kwargs):
        """Return an instance of the object based on the given
        identifier, or ``None`` if not found.

        The `ident` argument is a scalar or tuple of primary key
        column values in the order of the table def's primary key
        columns.

        The `entity_name` keyword argument may also be specified which
        further qualifies the underlying Mapper used to perform the
        query.
        """

        entity_name = kwargs.pop('entity_name', None)
        return self.query(class_, entity_name=entity_name).get(ident, **kwargs)

    def load(self, class_, ident, **kwargs):
        """Return an instance of the object based on the given
        identifier.

        If not found, raises an exception.  The method will **remove
        all pending changes** to the object already existing in the
        ``Session``.  The `ident` argument is a scalar or tuple of primary
        key columns in the order of the table def's primary key
        columns.

        The `entity_name` keyword argument may also be specified which
        further qualifies the underlying ``Mapper`` used to perform the
        query.
        """

        entity_name = kwargs.pop('entity_name', None)
        return self.query(class_, entity_name=entity_name).load(ident, **kwargs)

    def refresh(self, obj):
        """Reload the attributes for the given object from the
        database, clear any changes made.
        """

        self._validate_persistent(obj)
        if self.query(obj.__class__)._get(obj._instance_key, reload=True) is None:
            raise exceptions.InvalidRequestError("Could not refresh instance '%s'" % repr(obj))

    def expire(self, obj):
        """Mark the given object as expired.

        This will add an instrumentation to all mapped attributes on
        the instance such that when an attribute is next accessed, the
        session will reload all attributes on the instance from the
        database.
        """

        for c in [obj] + list(_object_mapper(obj).cascade_iterator('refresh-expire', obj)):
            self._expire_impl(c)

    def _expire_impl(self, obj):
        self._validate_persistent(obj)

        def exp():
            if self.query(obj.__class__)._get(obj._instance_key, reload=True) is None:
                raise exceptions.InvalidRequestError("Could not refresh instance '%s'" % repr(obj))

        attribute_manager.trigger_history(obj, exp)

    def is_expired(self, obj, unexpire=False):
        """Return True if the given object has been marked as expired."""

        ret = attribute_manager.has_trigger(obj)
        if ret and unexpire:
            attribute_manager.untrigger_history(obj)
        return ret

    def expunge(self, object):
        """Remove the given `object` from this ``Session``.

        This will free all internal references to the object.
        Cascading will be applied according to the *expunge* cascade
        rule.
        """
        self._validate_persistent(object)
        for c in [object] + list(_object_mapper(object).cascade_iterator('expunge', object)):
            if c in self:
                self.uow._remove_deleted(c)
                self._unattach(c)

    def save(self, object, entity_name=None):
        """Add a transient (unsaved) instance to this ``Session``.

        This operation cascades the `save_or_update` method to
        associated instances if the relation is mapped with
        ``cascade="save-update"``.

        The `entity_name` keyword argument will further qualify the
        specific ``Mapper`` used to handle this instance.
        """

        self._save_impl(object, entity_name=entity_name)
        _object_mapper(object).cascade_callable('save-update', object,
                                                lambda c, e:self._save_or_update_impl(c, e),
                                                halt_on=lambda c:c in self)

    def update(self, object, entity_name=None):
        """Bring the given detached (saved) instance into this
        ``Session``.

        If there is a persistent instance with the same identifier
        already associated with this ``Session``, an exception is thrown.

        This operation cascades the `save_or_update` method to
        associated instances if the relation is mapped with
        ``cascade="save-update"``.
        """

        self._update_impl(object, entity_name=entity_name)
        _object_mapper(object).cascade_callable('save-update', object,
                                                lambda c, e:self._save_or_update_impl(c, e),
                                                halt_on=lambda c:c in self)

    def save_or_update(self, object, entity_name=None):
        """Save or update the given object into this ``Session``.

        The presence of an `_instance_key` attribute on the instance
        determines whether to ``save()`` or ``update()`` the instance.
        """

        self._save_or_update_impl(object, entity_name=entity_name)
        _object_mapper(object).cascade_callable('save-update', object,
                                                lambda c, e:self._save_or_update_impl(c, e),
                                                halt_on=lambda c:c in self)

    def _save_or_update_impl(self, object, entity_name=None):
        key = getattr(object, '_instance_key', None)
        if key is None:
            self._save_impl(object, entity_name=entity_name)
        else:
            self._update_impl(object, entity_name=entity_name)

    def delete(self, object):
        """Mark the given instance as deleted.

        The delete operation occurs upon ``flush()``.
        """

        for c in [object] + list(_object_mapper(object).cascade_iterator('delete', object)):
            self.uow.register_deleted(c)

    def merge(self, object, entity_name=None, _recursive=None):
        """Copy the state of the given `object` onto the persistent
        object with the same identifier.

        If there is no persistent instance currently associated with
        the session, it will be loaded.  Return the persistent
        instance. If the given instance is unsaved, save a copy of and
        return it as a newly persistent instance. The given instance
        does not become associated with the session.

        This operation cascades to associated instances if the
        association is mapped with ``cascade="merge"``.
        """

        if _recursive is None:
            _recursive = util.Set()
        if entity_name is not None:
            mapper = _class_mapper(object.__class__, entity_name=entity_name)
        else:
            mapper = _object_mapper(object)
        if mapper in _recursive or object in _recursive:
            return None
        _recursive.add(mapper)
        _recursive.add(object)
        try:
            key = getattr(object, '_instance_key', None)
            if key is None:
                merged = mapper._create_instance(self)
            else:
                if key in self.identity_map:
                    merged = self.identity_map[key]
                else:
                    merged = self.get(mapper.class_, key[1])
                    if merged is None:
                        raise exceptions.AssertionError("Instance %s has an instance key but is not persisted" % mapperutil.instance_str(object))
            for prop in mapper.props.values():
                prop.merge(self, object, merged, _recursive)
            if key is None:
                self.save(merged, entity_name=mapper.entity_name)
            return merged
        finally:
            _recursive.remove(mapper)
            
    def identity_key(self, *args, **kwargs):
        """Get an identity key.

        Valid call signatures:

        * ``identity_key(class, ident, entity_name=None)``
        
          class
              mapped class (must be a positional argument)

          ident
              primary key, if the key is composite this is a tuple

          entity_name
              optional entity name

        * ``identity_key(instance=instance)``
        
          instance
              object instance (must be given as a keyword arg)

        * ``identity_key(class, row=row, entity_name=None)``
        
          class
              mapped class (must be a positional argument)

          row
              result proxy row (must be given as a keyword arg)

          entity_name
              optional entity name (must be given as a keyword arg)
        """

        if args:
            if len(args) == 1:
                class_ = args[0]
                try:
                    row = kwargs.pop("row")
                except KeyError:
                    ident = kwargs.pop("ident")
                entity_name = kwargs.pop("entity_name", None)
            elif len(args) == 2:
                class_, ident = args
                entity_name = kwargs.pop("entity_name", None)
            elif len(args) == 3:
                class_, ident, entity_name = args
            else:
                raise exceptions.ArgumentError("expected up to three "
                    "positional arguments, got %s" % len(args))
            if kwargs:
                raise exceptions.ArgumentError("unknown keyword arguments: %s"
                    % ", ".join(kwargs.keys()))
            mapper = _class_mapper(class_, entity_name=entity_name)
            if "ident" in locals():
                return mapper.identity_key_from_primary_key(ident)
            return mapper.identity_key_from_row(row)
        instance = kwargs.pop("instance")
        if kwargs:
            raise exceptions.ArgumentError("unknown keyword arguments: %s"
                % ", ".join(kwargs.keys()))
        mapper = _object_mapper(instance)
        return mapper.identity_key_from_instance(instance)

    def _save_impl(self, object, **kwargs):
        if hasattr(object, '_instance_key'):
            if not self.identity_map.has_key(object._instance_key):
                raise exceptions.InvalidRequestError("Instance '%s' is a detached instance "
                                                     "or is already persistent in a "
                                                     "different Session" % repr(object))
        else:
            m = _class_mapper(object.__class__, entity_name=kwargs.get('entity_name', None))

            # this would be a nice exception to raise...however this is incompatible with a contextual
            # session which puts all objects into the session upon construction.
            #if m._is_orphan(object):
            #    raise exceptions.InvalidRequestError("Instance '%s' is an orphan, "
            #                                         "and must be attached to a parent "
            #                                         "object to be saved" % (repr(object)))

            m._assign_entity_name(object)
            self._register_pending(object)

    def _update_impl(self, object, **kwargs):
        if self._is_attached(object) and object not in self.deleted:
            return
        if not hasattr(object, '_instance_key'):
            raise exceptions.InvalidRequestError("Instance '%s' is not persisted" % repr(object))
        self._attach(object)

    def _register_pending(self, obj):
        self._attach(obj)
        self.uow.register_new(obj)

    def _register_persistent(self, obj):
        self._attach(obj)
        self.uow.register_clean(obj)

    def _register_deleted(self, obj):
        self._attach(obj)
        self.uow.register_deleted(obj)

    def _attach(self, obj):
        """Attach the given object to this ``Session``."""

        if getattr(obj, '_sa_session_id', None) != self.hash_key:
            old = getattr(obj, '_sa_session_id', None)
            if old is not None and _sessions.has_key(old):
                raise exceptions.InvalidRequestError("Object '%s' is already attached "
                                                     "to session '%s' (this is '%s')" %
                                                     (repr(obj), old, id(self)))

                # auto-removal from the old session is disabled.  but if we decide to
                # turn it back on, do it as below: gingerly since _sessions is a WeakValueDict
                # and it might be affected by other threads
                #try:
                #    sess = _sessions[old]
                #except KeyError:
                #    sess = None
                #if sess is not None:
                #    sess.expunge(old)
            key = getattr(obj, '_instance_key', None)
            if key is not None:
                self.identity_map[key] = obj
            obj._sa_session_id = self.hash_key

    def _unattach(self, obj):
        if not self._is_attached(obj):
            raise exceptions.InvalidRequestError("Instance '%s' not attached to this Session" % repr(obj))
        del obj._sa_session_id

    def _validate_persistent(self, obj):
        """Validate that the given object is persistent within this
        ``Session``.
        """

        self.uow._validate_obj(obj)

    def _is_attached(self, obj):
        return getattr(obj, '_sa_session_id', None) == self.hash_key

    def __contains__(self, obj):
        return self._is_attached(obj) and (obj in self.uow.new or self.identity_map.has_key(obj._instance_key))

    def __iter__(self):
        return iter(list(self.uow.new) + self.uow.identity_map.values())

    def _get(self, key):
        return self.identity_map[key]

    def has_key(self, key):
        return self.identity_map.has_key(key)

    dirty = property(lambda s:s.uow.locate_dirty(),
                     doc="A ``Set`` of all objects marked as 'dirty' within this ``Session``")

    deleted = property(lambda s:s.uow.deleted,
                       doc="A ``Set`` of all objects marked as 'deleted' within this ``Session``")

    new = property(lambda s:s.uow.new,
                   doc="A ``Set`` of all objects marked as 'new' within this ``Session``.")

    identity_map = property(lambda s:s.uow.identity_map,
                            doc="A dictionary consisting of all objects "
                            "within this ``Session`` keyed to their `_instance_key` value.")

    def import_instance(self, *args, **kwargs):
        """Deprecated. A synynom for ``merge()``."""

        return self.merge(*args, **kwargs)

# this is the AttributeManager instance used to provide attribute behavior on objects.
# to all the "global variable police" out there:  its a stateless object.
attribute_manager = unitofwork.attribute_manager

# this dictionary maps the hash key of a Session to the Session itself, and
# acts as a Registry with which to locate Sessions.  this is to enable
# object instances to be associated with Sessions without having to attach the
# actual Session object directly to the object instance.
_sessions = weakref.WeakValueDictionary()

def object_session(obj):
    """Return the ``Session`` to which the given object is bound, or ``None`` if none."""

    hashkey = getattr(obj, '_sa_session_id', None)
    if hashkey is not None:
        return _sessions.get(hashkey)
    return None

unitofwork.object_session = object_session
from sqlalchemy.orm import mapper
mapper.attribute_manager = attribute_manager



# -*- coding: UTF-8 -*-

__revision__ = '$Id: PluginMovieIMDB-de.py 1042 2008-11-15 21:59:58Z mikej06 $'

# Copyright (c) 2007 Michael Jahn
#
# This program is free software; you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation; either version 2 of the License, or
# (at your option) any later version.
#
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU Library General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with this program; if not, write to the Free Software
# 51 Franklin Street, Fifth Floor, Boston, MA  02110-1301, USA

# You may use and distribute this software under the terms of the
# GNU General Public License, version 2 or later

import gutils, movie
import string, re

plugin_name         = 'IMDb-de'
plugin_description  = 'Internet Movie Database German'
plugin_url          = 'german.imdb.com'
plugin_language     = _('German')
plugin_author       = 'Michael Jahn'
plugin_author_email = 'mikej06@hotmail.com'
plugin_version      = '1.3'

class Plugin(movie.Movie):
	def __init__(self, id):
		self.encode = 'iso8859-1'
		self.movie_id = id
		self.url = "http://german.imdb.com/title/tt%s" % str(self.movie_id)

	def initialize(self):
		self.cast_page = self.open_page(url=self.url + '/fullcredits')
		self.plot_page = self.open_page(url=self.url + '/plotsummary')
		# correction of all &#xxx entities
		self.page = self.page.decode(self.encode)
		self.page = gutils.convert_entities(self.page)
		self.page = self.page.encode(self.encode)
		self.cast_page = self.cast_page.decode(self.encode)
		self.cast_page = gutils.convert_entities(self.cast_page)
		self.cast_page = self.cast_page.encode(self.encode)
		self.plot_page = self.plot_page.decode(self.encode)
		self.plot_page = gutils.convert_entities(self.plot_page)
		self.plot_page = self.plot_page.encode(self.encode)

	def get_image(self):
		tmp = string.find(self.page, 'a name="poster"')
		if tmp == -1:		# poster not available
			self.image_url = ''
		else:
			self.image_url = gutils.trim(self.page[tmp:], 'src="', '"')

	def get_o_title(self):
		self.o_title = gutils.trim(self.page, '<h1>', '<span')

	def get_title(self):
		self.title = gutils.trim(self.page, '<h1>', '<span')
		elements = string.split(self.regextrim(self.page, '<h5>(Alternativ|Auch bekannt als):', '</div>'), '<i class="transl"')
		if len(elements) > 1:
			for element in elements:
				tmp = gutils.before(gutils.trim(element, '>', '[de]'), '(')
				if tmp <> '':
					self.title = tmp
					break

	def get_director(self):
		self.director = gutils.trim(self.page,'<h5>Regie</h5>', '<br/>')
		if self.director == '':
			self.director = gutils.trim(self.page,'<h5>Regisseur:</h5>', '</div>')
		self.director = self.__before_more(self.director)
		self.director = self.director.replace('<br/>', ', ')
		self.director = gutils.clean(self.director)
		self.director = re.sub(',$', '', self.director)

	def get_plot(self):
		self.plot = gutils.trim(self.page, '<h5>Kurzbeschreibung:</h5>', '</div>')
		self.plot = self.__before_more(self.plot)
		elements = string.split(self.plot_page, '<p class="plotpar">')
		if len(elements) > 1:
			self.plot = self.plot + '\n\n'
			elements[0] = ''
			for element in elements:
				if element != '':
					self.plot = self.plot + gutils.strip_tags(gutils.before(element, '</a>')) + '\n'

	def get_year(self):
		self.year = gutils.trim(self.page, '<a href="/Sections/Years/', '</a>')
		self.year = gutils.after(self.year, '">')

	def get_runtime(self):
		self.runtime = self.regextrim(self.page, '<h5>L[^n]+nge:</h5>', ' [Mm]in')

	def get_genre(self):
		self.genre = gutils.trim(self.page, '<h5>Genre:</h5>', '</div>')
		self.genre = self.__before_more(self.genre)

	def get_cast(self):
		self.cast = ''
		self.cast = gutils.trim(self.cast_page, '<table class="cast">', '</table>')
		if self.cast == '':
			self.cast = gutils.trim(self.page, '<table class="cast">', '</table>')
		self.cast = string.replace(self.cast, ' ... ', _(' as ').encode('utf8'))
		self.cast = string.replace(self.cast, '...', _(' as ').encode('utf8'))
		self.cast = string.replace(self.cast, '</tr><tr>', "\n")
		self.cast = string.replace(self.cast, '</tr><tr class="even">', "\n")
		self.cast = string.replace(self.cast, '</tr><tr class="odd">', "\n")
		self.cast = self.__before_more(self.cast)
		self.cast = re.sub('[ ]+', ' ', self.cast)

	def get_classification(self):
		self.classification = gutils.trim(gutils.trim(self.page, 'Altersfreigabe:', '</div>'), 'Germany:', '&')

	def get_studio(self):
		self.studio = gutils.trim(self.page, '<h5>Firma:</h5>', '</div>')
		self.studio = self.__before_more(self.studio)

	def get_o_site(self):
		self.o_site = ''

	def get_site(self):
		self.site = "http://german.imdb.com/title/tt%s" % self.movie_id

	def get_trailer(self):
		self.trailer = "http://german.imdb.com/title/tt%s/trailers" % self.movie_id

	def get_country(self):
		self.country = gutils.trim(self.page, '<h5>Land:</h5>', '</div>')
		self.country = self.__before_more(self.country)
		self.country = re.sub('[\n]+', '', self.country)
		self.country = re.sub('[ ]+', ' ', self.country)

	def get_rating(self):
		self.rating = gutils.trim(self.page, '<h5>Nutzer-Bewertung:</h5>', '/10')
		if self.rating:
			try:
				self.rating = str(float(gutils.clean(self.rating)))
			except:
				self.rating = ''

	def get_notes(self):
		self.notes = ''
		language = gutils.trim(self.page, '<h5>Sprache:</h5>', '</div>')
		language = gutils.strip_tags(language)
		language = re.sub('[\n]+', '', language)
		language = re.sub('[ ]+', ' ', language)
		language = language.rstrip()
		color = gutils.trim(self.page, '<h5>Farbe:</h5>', '</div>')
		color = gutils.strip_tags(color)
		color = re.sub('[\n]+', '', color)
		color = re.sub('[ ]+', ' ', color)
		color = re.sub('[ ]+$', '', color)
		color = color.rstrip()
		sound = gutils.trim(self.page, '<h5>Tonverfahren:</h5>', '</div>')
		sound = gutils.strip_tags(sound)
		sound = re.sub('[\n]+', '', sound)
		sound = re.sub('[ ]+', ' ', sound)
		sound = sound.rstrip()
		soundsplit = sound.split(' | ')
		if len(soundsplit) > 1:
			soundsplit.sort()
			sound = ''
			for elem in soundsplit:
				sound += elem + ' | '
			sound = sound[0:len(sound) - 3]
		tagline = gutils.trim(self.page, '<h5>Werbezeile:</h5>', '</div>')
		tagline = self.__before_more(tagline)
		tagline = gutils.strip_tags(tagline)
		tagline = re.sub('[\n]+', '', tagline)
		tagline = re.sub('[ ]+', ' ', tagline)
		tagline = tagline.rstrip()
		if len(language)>0:
			self.notes = "%s: %s\n" %(_('Language').encode('utf8'), language)
		if len(sound)>0:
			self.notes += "%s: %s\n" %(gutils.strip_tags(_('<b>Audio</b>').encode('utf8')), sound)
		if len(color)>0:
			self.notes += "%s: %s\n" %(_('Color').encode('utf8'), color)
		if len(tagline)>0:
			self.notes += "%s: %s\n" %('Tagline', tagline)
	
	def __before_more(self, data):
		tmp = string.find(data, '>mehr<')
		if tmp>0:
			data = data[:tmp] + '>'
		return data

	def regextrim(self,text,key1,key2):
		obj = re.search(key1, text)
		if obj is None:
			return ''
		else:
			p1 = obj.end()
		obj = re.search(key2, text[p1:])
		if obj is None:
			return ''
		else:
			p2 = p1 + obj.start()
		return text[p1:p2]

class SearchPlugin(movie.SearchMovie):
	PATTERN = re.compile(r"""<a href=['"]/title/tt([0-9]+)/["']>(.*?)(</td>|</A>)""", re.IGNORECASE)
	PATTERN_POWERSEARCH = re.compile(r"""Here are the [0-9]+ matching titles""")

	def __init__(self):
		self.original_url_search	= 'http://german.imdb.com/find?more=tt&q='
		self.translated_url_search	= 'http://german.imdb.com/find?more=tt&q='
		self.encode = 'iso8859-1'
		self.remove_accents = False

	def search(self,parent_window):
		self.open_search(parent_window)
		tmp = gutils.trim(self.page, ' angezeigt)', ' Treffergenauigkeit')
		if tmp == '':
			if self.PATTERN_POWERSEARCH.search(self.page) is None:
				self.page = ''
		else:
			self.page = tmp 
		# correction of all &#xxx entities
		self.page = self.page.decode(self.encode)
		self.page = gutils.convert_entities(self.page)
		self.page = self.page.encode(self.encode)
		return self.page

	def get_searches(self):
		elements = string.split(self.page, '<tr>')
		if len(elements) < 2:
			elements = string.split(self.page, '<TR>')

		if len(elements):
			for element in elements[1:]:
				match = self.PATTERN.findall(element)
				for entry in match:
					tmp  = gutils.clean(entry[1])
					self.ids.append(entry[0])
					self.titles.append(tmp)

#
# Plugin Test
#
class SearchPluginTest(SearchPlugin):
	#
	# Configuration for automated tests:
	# dict { movie_id -> [ expected result count for original url, expected result count for translated url ] }
	#
	test_configuration = {
		'Rocky Balboa'			: [ 3, 13 ],
		'Ein glückliches Jahr'	: [ 1, 30 ]
	}

class PluginTest:
	#
	# Configuration for automated tests:
	# dict { movie_id -> dict { arribute -> value } }
	#
	# value: * True/False if attribute only should be tested for any value
	#        * or the expected value
	#
	test_configuration = {
		'0479143' : { 
			'title' 			: 'Rocky Balboa',
			'o_title' 			: 'Rocky Balboa',
			'director'			: 'Sylvester Stallone',
			'plot' 				: True,
			'cast'				: 'Sylvester Stallone' + _(' as ') + 'Rocky Balboa\n\
Burt Young' + _(' as ') + 'Paulie Panina\n\
Antonio Tarver' + _(' as ') + 'Mason \'The Line\' Dixon\n\
Geraldine Hughes' + _(' as ') + 'Marie\n\
Milo Ventimiglia' + _(' as ') + 'Robert Balboa Jr.\n\
Tony Burton' + _(' as ') + 'Duke\n\
A.J. Benza' + _(' as ') + 'L.C.\n\
James Francis Kelly III' + _(' as ') + 'Steps\n\
Talia Shire' + _(' as ') + 'Adrian (archive footage)\n\
Lou DiBella' + _(' as ') + 'Himself\n\
Mike Tyson' + _(' as ') + 'Himself\n\
Henry G. Sanders' + _(' as ') + 'Martin\n\
Pedro Lovell' + _(' as ') + 'Spider Rico\n\
Ana Gerena' + _(' as ') + 'Isabel\n\
Angela Boyd' + _(' as ') + 'Angie\n\
Louis Giansante' + _(' as ') + 'Bar Thug\n\
Maureen Schilling' + _(' as ') + 'Lucky\'s Bartender\n\
Lahmard J. Tate' + _(' as ') + 'X-Cell (as Lahmard Tate)\n\
Woody Paige' + _(' as ') + 'ESPN Commentator (as Woodrow W. Paige)\n\
Skip Bayless' + _(' as ') + 'ESPN Commentator\n\
Jay Crawford' + _(' as ') + 'ESPN Commentator\n\
Brian Kenny' + _(' as ') + 'ESPN Host\n\
Dana Jacobson' + _(' as ') + 'ESPN Host\n\
Charles Johnson' + _(' as ') + 'ESPN Host (as Chuck Johnson)\n\
James Binns' + _(' as ') + 'Commissioner (as James J. Binns)\n\
Johnnie Hobbs Jr.' + _(' as ') + 'Commissioner\n\
Barney Fitzpatrick' + _(' as ') + 'Commissioner\n\
Jim Lampley' + _(' as ') + 'HBO Commentator\n\
Larry Merchant' + _(' as ') + 'HBO Commentator\n\
Max Kellerman' + _(' as ') + 'HBO Commentator\n\
LeRoy Neiman' + _(' as ') + 'Himself\n\
Bert Randolph Sugar' + _(' as ') + 'Ring Magazine Reporter\n\
Bernard Fernández' + _(' as ') + 'Boxing Association of America Writer (as Bernard Fernandez)\n\
Gunnar Peterson' + _(' as ') + 'Weightlifting Trainer\n\
Yahya' + _(' as ') + 'Dixon\'s Opponent\n\
Marc Ratner' + _(' as ') + 'Weigh-In Official\n\
Anthony Lato Jr.' + _(' as ') + 'Rocky\'s Inspector\n\
Jack Lazzarado' + _(' as ') + 'Dixon\'s Inspector\n\
Michael Buffer' + _(' as ') + 'Ring Announcer\n\
Joe Cortez' + _(' as ') + 'Referee\n\
Carter Mitchell' + _(' as ') + 'Shamrock Foreman\n\
Vinod Kumar' + _(' as ') + 'Ravi\n\
Fran Pultro' + _(' as ') + 'Father at Restaurant\n\
Frank Stallone' + _(' as ') + 'Dinner Patron (as Frank Stallone Jr.)\n\
Jody Giambelluca' + _(' as ') + 'Dinner Patron\n\
Tobias Segal' + _(' as ') + 'Robert\'s Friend\n\
Tim Carr' + _(' as ') + 'Robert\'s Friend\n\
Matt Frack' + _(' as ') + 'Robert\'s Friend #3\n\
Paul Dion Monte' + _(' as ') + 'Robert\'s Friend\n\
Kevin King Templeton' + _(' as ') + 'Robert\'s Friend (as Kevin King-Templeton)\n\
Robert Michael Kelly' + _(' as ') + 'Mr. Tomilson\n\
Rick Buchborn' + _(' as ') + 'Rocky Fan\n\
Nick Baker' + _(' as ') + 'Irish Pub Bartender\n\
Don Sherman' + _(' as ') + 'Andy\n\
Stu Nahan' + _(' as ') + 'Computer Fight Commentator (voice)\n\
Gary Compton' + _(' as ') + 'Security Guard\n\
übrige Besetzung in alphabetischer Reihenfolge:\n\
Ricky Cavazos' + _(' as ') + 'Boxing Spectator (uncredited)\n\
David Kneeream' + _(' as ') + 'Adrian\'s Patron (uncredited)\n\
Dolph Lundgren' + _(' as ') + 'Captain Ivan Drago (archive footage) (uncredited)\n\
Burgess Meredith' + _(' as ') + 'Mickey Goldmill (archive footage) (uncredited)\n\
Keith Moyer' + _(' as ') + 'Bar Patron (uncredited)\n\
Mr. T' + _(' as ') + 'Clubber Lang (archive footage) (uncredited)',
			'country'			: 'USA',
			'genre'				: 'Action | Sport',
			'classification'	: False,
			'studio'			: 'Metro-Goldwyn-Mayer (MGM)',
			'o_site'			: False,
			'site'				: 'http://german.imdb.com/title/tt0479143',
			'trailer'			: 'http://german.imdb.com/title/tt0479143/trailers',
			'year'				: 2006,
			'notes'				: _('Language') + ': Englisch | Spanisch\n'\
+ _('Audio') + ': DTS | Dolby Digital | SDDS\n'\
+ _('Color') + ': Farbe\n\
Tagline: It ain\'t over \'til it\'s over.',
			'runtime'			: 102,
			'image'				: True,
			'rating'			: 7
		},
		'0069815' : { 
			'title' 			: 'Glückliches Jahr, Ein',
			'o_title' 			: 'Bonne année, La',
			'director'			: 'Claude Lelouch',
			'plot' 				: True,
			'cast'				: 'Lino Ventura' + _(' as ') + 'Simon\n\
Françoise Fabian' + _(' as ') + 'Françoise\n\
Charles Gérard' + _(' as ') + 'Charlot\n\
André Falcon' + _(' as ') + 'Le bijoutier\n\
Mireille Mathieu' + _(' as ') + 'Herself / Elle-même\n\
Lilo' + _(' as ') + 'Madame Félix\n\
Claude Mann' + _(' as ') + 'L\'intellectuel\n\
Frédéric de Pasquale' + _(' as ') + 'L\'amant parisien\n\
Gérard Sire' + _(' as ') + 'Le directeur de la prison\n\
Silvano Tranquilli' + _(' as ') + 'L\'amant italien\n\
André Barello\n\
Michel Bertay\n\
Norman de la Chesnaye\n\
Pierre Edeline\n\
Pierre Pontiche\n\
Michou' + _(' as ') + 'Himself\n\
Bettina Rheims' + _(' as ') + 'La jeune vendeuse\n\
Joseph Rythmann\n\
Georges Staquet\n\
Jacques Villedieu\n\
Harry Walter\n\
übrige Besetzung in alphabetischer Reihenfolge:\n\
Anouk Aimée' + _(' as ') + 'Une femme (archive footage) (uncredited)\n\
Elie Chouraqui' + _(' as ') + '(uncredited)\n\
Rémy Julienne' + _(' as ') + 'Chauffeur de taxi (uncredited)\n\
Jean-Louis Trintignant' + _(' as ') + 'Un homme (archive footage) (uncredited)',
			'country'			: 'France | Italy',
			'genre'				: 'Komödie',
			'classification'	: False,
			'studio'			: 'Les Films 13',
			'o_site'			: False,
			'site'				: 'http://german.imdb.com/title/tt0069815',
			'trailer'			: 'http://german.imdb.com/title/tt0069815/trailers',
			'year'				: 1973,
			'notes'				: _('Language') + ': Französisch\n'\
+ _('Audio') + ': Mono\n'\
+ _('Color') + ': Farbe (Eastmancolor)',
			'runtime'			: 90,
			'image'				: True,
			'rating'			: 7
		},
	}



# -*- coding: UTF-8 -*-

__revision__ = '$Id: CSV.py 1040 2008-11-15 21:13:49Z mikej06 $'

###########################################################################
#    Copyright (C) 2006-2007 by Jessica Katharina Parth                         
#    <Jessica.K.P@women-at-work.org>                                       
#
# Copyright: See COPYING file that comes with this distribution
#
###########################################################################

from plugins.imp import ImportPlugin as IP
import gtk
import gtk.glade
import os
import pygtk
import sys
import gutils
import string


def digits_only(s):
	import string, re
	_match = re.compile(r"\d+")
	try:
		s = reduce( string.join, _match.findall(s) )
	except:
		s = '0'
	return s
	
def letters_only(s):
	import string, re
	_match = re.compile(r"\D+")
	try:
		s = reduce( string.join, _match.findall(s) )
	except:
		s = s
	return s
	
class ImportPlugin(IP):
	description	= _("Full CSV list import plugin")
	author		= "Jessica Katharina Parth"
	email		= "Jessica.K.P@women-at-work.org"
	version		= "0.3"
	file_filters	= '*.[cC][sS][vV]'
	mime_types	= ('text/comma-separated-values', 'text/csv', 'application/csv')

	def initialize(self):
		if not IP.initialize(self):
			return False
		# glade
		gf = os.path.join(self.locations['glade'], 'importcsv.glade')
		# try to open the glade file
		try:
			self.gtk = gtk.glade.XML(gf)
		except:
			self.debug.show("Glade-file %s can not be loaded." % gf)
			return False
		# open gtk window
		self.gtk.get_widget('d_import').set_transient_for( self.widgets['window'] )
		
		# simple string lists
		self.tv_csv = self.gtk.get_widget('tv_csv')
		self.tv_assigned = self.gtk.get_widget('tv_assigned')
		self.tv_griffith = self.gtk.get_widget('tv_griffith')
		
		# 1st list
		self.ls_csv = gtk.ListStore(str)
		self.tv_csv.set_model(self.ls_csv)
		renderer = gtk.CellRendererText()
		column = gtk.TreeViewColumn("none", renderer, text=0)
		self.tv_csv.append_column(column)
		
		# 2nd list 
		self.ls_griffith = gtk.ListStore(str,str)
		self.tv_griffith.set_model(self.ls_griffith)
		renderer = gtk.CellRendererText()
		column = gtk.TreeViewColumn("none", renderer, text=0)
		column.set_visible(False)
		self.tv_griffith.append_column(column)
		column = gtk.TreeViewColumn("none", renderer, text=1)
		self.tv_griffith.append_column(column)
		self.set_griffith_fields()
		
		# 3rd list
		self.ls_assigned = gtk.ListStore(str,str,str)
		self.tv_assigned.set_model(self.ls_assigned)
		renderer = gtk.CellRendererText()
		column = gtk.TreeViewColumn("none", renderer, text=0)
		self.tv_assigned.append_column(column)
		# add the columns for internal information handling and hide them
		column = gtk.TreeViewColumn("none", renderer, text=1)
		column.set_visible(False)
		self.tv_assigned.append_column(column)
		column = gtk.TreeViewColumn("none", renderer, text=2)
		column.set_visible(False)
		self.tv_assigned.append_column(column)
	
		# hide tabs
		self.nb_pages = self.gtk.get_widget('nb_pages')
		self.nb_pages.get_nth_page(1).hide()
		self.nb_pages.connect("switch-page", self._on_page_changed)
		
		# Events
		# Buttons
		self.b_cancel = self.gtk.get_widget("b_cancel")
		self.b_cancel.connect("clicked", self._clicked)
		
		self.b_next = self.gtk.get_widget("b_next")
		self.b_next.connect("clicked", self._clicked)
		
		self.b_back = self.gtk.get_widget("b_back")
		self.b_back.connect("clicked", self._clicked)
		
		self.b_add = self.gtk.get_widget("b_add")
		self.b_add.connect("clicked", self._clicked)
		
		self.b_del = self.gtk.get_widget("b_del")
		self.b_del.connect("clicked", self._clicked)
		
		# Treeviews
		self.tv_griffith.connect("row_activated", self._on_row_activated)
		self.tv_griffith.connect("cursor_changed", self._on_cursor_changed)
		self.tv_csv.connect("row_activated", self._on_row_activated)
		self.tv_csv.connect("cursor_changed", self._on_cursor_changed)
		self.tv_assigned.connect("cursor_changed", self._on_cursor_changed)
		
		self.gtk.get_widget('e_lineterminator').set_active(0)
		
		self.selected_griffith = None
		self.selected_csv = None
		self.current_csv_row = 0
		self.csv_header = None
		return True
	
	def set_griffith_fields(self):
		# 2nd list
		sorted_list = ( "number", "title", "o_title", "director", "year", "country",
				"cast", "studio", "plot", "runtime", "genre", "classification",
				"site", "o_site", "trailer", "image", "seen", "loaned", "notes",
				"rating", "movie_id", "collection_id", "volume_id", "medium_id",
				"vcodec_id", "color", "cond", "layers", "region", "media_num" )
		# sort the list and add field and translated field-name
		for sorted in sorted_list:
			for name in self.fields_to_import:
				if sorted == name:
					iterator = self.ls_griffith.append()
					self.ls_griffith.set_value(iterator, 0, name)
					self.ls_griffith.set_value(iterator, 1, self.fields[name])

	def create_import_table(self):
		self.import_table = {}
		item = self.ls_assigned.get_iter_first()
		while item is not None:
			self.import_table[self.ls_assigned.get_value(item,1)] = self.ls_assigned.get_value(item,2)
			item = self.ls_assigned.iter_next(item)
			
	def _on_page_changed(self, notebook, page, page_num):
		if page_num == 0:
			self.b_back.set_sensitive(False)
		if page_num == 1:
			self.b_back.set_sensitive(True)
			self.open_source()
			
		
	def _clicked(self, widget, event=None, data=None):
		if widget == self.b_cancel:
			self.gtk.get_widget('d_import').hide()
			self.gtk.get_widget('d_import').response(gtk.RESPONSE_CANCEL)
			
		if widget == self.b_back:
			if self.nb_pages.get_current_page() == 1:
				self.nb_pages.prev_page()

		if widget == self.b_next:
			if self.nb_pages.get_current_page() == 0:
				self.nb_pages.get_nth_page(1).show()
				self.nb_pages.next_page()
			else:
				if self.nb_pages.get_current_page() == 1:
					# test if at least one field is assigned
					if self.ls_assigned.get_iter_first() is not None:
						# prepare tabelle for import
						self.create_import_table()
							
						# hide everything
						self.gtk.get_widget('d_import').hide()
						self.gtk.get_widget('d_import').response(gtk.RESPONSE_OK)
					else:
						gutils.info(self.gtk, _("Please assign at least one field first!"), self.gtk.get_widget('d_import') )
						
		if widget == self.b_add:
			iterator = self.ls_assigned.append()
			self.ls_assigned.set_value(iterator, 0, "%s > %s" % (self.selected_csv, self.fields[self.selected_griffith]) )
			# add information for the import tabelle
			self.ls_assigned.set_value(iterator, 1, self.selected_griffith )
			self.ls_assigned.set_value(iterator, 2, str(self.csv_header.index(self.selected_csv)) )
			self.ls_griffith.remove(self.iter_griffith)
			self.selected_griffith = None
			
			self.b_add.set_sensitive(False)
		
		if widget == self.b_del:
			# re-add field to griffith
			field_name = self.ls_assigned.get_value(self.iter_assigned,1)

			iterator = self.ls_griffith.append()
			self.ls_griffith.set_value(iterator, 0, field_name)
			self.ls_griffith.set_value(iterator, 1, self.fields[field_name])
			
			# remove assigned row
			self.ls_assigned.remove(self.iter_assigned)
			
			self.b_del.set_sensitive(False)
			
					
	def _on_row_activated(self, treeview, path, view_column, data = None):
		# get selected rows from both treeviews/lists
		if treeview == self.tv_griffith:
			self.iter_griffith = self.ls_griffith.get_iter(path)
		
			if self.iter_griffith:
				self.selected_griffith = self.ls_griffith.get_value(self.iter_griffith,0)
		
		if treeview == self.tv_csv:
			iter = self.ls_csv.get_iter(path)
		
			if iter:
				self.selected_csv = self.ls_csv.get_value(iter,0)
				
		# enable add button if both lists have a selected item
		if self.selected_griffith is not None and self.selected_csv is not None:
			self.b_add.set_sensitive(True)
		else:
			self.b_add.set_sensitive(False)
		
	def _on_cursor_changed(self, widget, data1 = None, data2 = None):
		# get selected rows from both treeviews/lists
		selection = widget.get_selection()
		(model,iter) = selection.get_selected()
			
		if widget == self.tv_griffith:
			self.iter_griffith = iter
			if self.iter_griffith:
				self.selected_griffith = str(model.get_value(self.iter_griffith, 0))
			else:
				self.selected_griffith = None
		
		if widget == self.tv_csv:
			if iter:
				self.selected_csv = str(model.get_value(iter, 0))
			else:
				self.selected_csv = None
				
		if widget == self.tv_assigned:
			self.iter_assigned = iter
			if self.iter_assigned:
				self.b_del.set_sensitive(True)
			else:
				self.b_del.set_sensitive(False)
				
		# enable add button if both lists have a selected item
		if self.selected_griffith is not None and self.selected_csv is not None:
			self.b_add.set_sensitive(True)
		else:
			self.b_add.set_sensitive(False)
			
	def open_source(self):
		import csv, codecs, os
		# get user values for converting/opening the csv-file
		self.start_row = int(digits_only( self.gtk.get_widget('e_startrow').get_text() ))
		encoding = self.gtk.get_widget('e_encoding').get_active_text()
		encoding = encoding[:string.find( encoding, ' ' )]
		delimiter = self.gtk.get_widget('e_delimiter').get_text()
		if delimiter == '':
			delimiter = ","
		# quotechar
		quotechar =  self.gtk.get_widget('e_quotechar').get_text()
		if quotechar == '':
			quotechar == '"'
		# lineterminator
		active = self.gtk.get_widget('e_lineterminator').get_active()
		# default for none selected and the same for linux and macintosh
		lineterminator = '\r'
		# windows lineterminator
		if active == 1:
			lineterminator = '\r\n'
		
		# open file
		try:
			self.data = csv.reader(codecs.open(self.__source_name, 'r', encoding), dialect='excel', quotechar=quotechar, delimiter=delimiter, lineterminator = lineterminator)

			# get the first line in csv file for the field names
			self.csv_header = self.data.next()
			
			# if the user wants to import line 0 then we have to open it again 
			if self.start_row == 0:
				self.data = csv.reader(codecs.open(self.__source_name, 'r', encoding), dialect='excel', quotechar=quotechar, delimiter=delimiter, lineterminator = lineterminator)
			
		
			# fill the found csv-headers in the simple string list
			self.ls_csv.clear()
			for name in self.csv_header:
				iterator = self.ls_csv.append()
				self.ls_csv.set_value(iterator, 0, name)
			return True
		except:
			gutils.info(self.gtk, _("Can't open the file %s") % self.__source_name, self.gtk.get_widget('d_import') )
			return False
			

	def set_source(self, name):
		import os
		# source _dependent_ initialization goes here
		if name is None or not os.path.isfile(name):
			return False
		self.__source_name = name
		# auto-detect file-encoding (optional)
		try:
			from chardet.universaldetector import UniversalDetector
			detector = UniversalDetector()
			detector.reset()
			lines = 0
			for line in file(self.__source_name, 'rb'):
				detector.feed(line)
				lines += 1
				if detector.done or lines == 50:
					break
			detector.close()
			encoding = string.replace( string.lower( detector.result['encoding'] ), '-', '' )
		except:
			encoding = 'utf_8'
		# remove - and _ for better detection
		encoding = string.replace( encoding, '_', '' )
		
		model	= self.gtk.get_widget('e_encoding').get_model()
		itempos	= 0
		for item in model:
			pos1 = string.find( string.replace( string.lower(str(item[0])), '_', '' ) , encoding )
			if pos1 == 0:
				break
			itempos += 1
		self.gtk.get_widget('e_encoding').set_active(itempos)
		
		# run dialog
		response = self.gtk.get_widget('d_import').run()
		if response == gtk.RESPONSE_OK:
			return True
		else:
			return False

	def count_movies(self):
		i = 0
		try:
			import csv
			data = csv.reader(open(self.__source_name))
			while data.next():
				i += 1
		except:
			return i
	
	def get_movie_details(self):
		try:
			item = self.data.next()
		except:
			return None
		if item is None:
			return None
		import copy
		# start with the right line
		self.current_csv_row += 1
		if (self.current_csv_row ) < self.start_row:
			return None
		
		# assign the keys
		t_movies = copy.deepcopy(self.import_table)

		# values are overwritten here with the imports
		for field in self.import_table:
			try:
				# some minor fixes to the import so it fits the griffith variable types
				if field == 'year' or field == 'runtime' or field == 'media_num' or field == 'number'  or field == 'volume_id':
					t_movies[field] = int( digits_only( item[ int(self.import_table[field]) ] ) )
				elif field == 'seen' or field == 'loaned':
					t_movies[field] = bool( item[ int(self.import_table[field]) ] )
				elif field == 'country':
					t_movies[field] = letters_only( item[ int(self.import_table[field]) ] )
				elif field == 'cast':
					try:
						if item[ int(self.import_table[field]) ].index(", ") != -1:
							t_movies[field] = string.replace( item[ int(self.import_table[field]) ], ", ", "\n" )
					except:
						t_movies[field] = string.replace( item[ int(self.import_table[field]) ], ",", "\n" )
				else:
					# 1:1 import
					t_movies[field] = item[ int(self.import_table[field]) ]
			except:
				# error field can't be imported
				t_movies.pop(field)
		
		return t_movies

	def clear(self):
		IP.clear(self)
		self.nb_pages.get_nth_page(1).hide()
		self.csv_header = None
		self.ls_assigned.clear()
		self.ls_griffith.clear()
		# add default griffith fields again
		self.set_griffith_fields()

	def destroy(self):
		self.gtk.get_widget('d_import').destroy()



# -*- coding: utf-8 -*-

__revision__ = '$Id: PluginMovieWP.py 1040 2008-11-15 21:13:49Z mikej06 $'

# Copyright (c) 2005-2007 Piotr Ożarowski
#
# This program is free software; you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation; either version 2 of the License, or
# (at your option) any later version.
#
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU Library General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with this program; if not, write to the Free Software
# 51 Franklin Street, Fifth Floor, Boston, MA  02110-1301, USA

# You may use and distribute this software under the terms of the
# GNU General Public License, version 2 or later

import gutils
import movie,string
import re

plugin_name         = 'Wirtualna Polska'
plugin_description  = 'Serwis filmowy Wirtualnej Polski'
plugin_url          = 'www.film.wp.pl'
plugin_language     = _('Polish')
plugin_author       = 'Piotr Ożarowski'
plugin_author_email = '<ozarow+griffith@gmail.com>'
plugin_version      = '2.3'

class Plugin(movie.Movie):
	def __init__(self, id):
		self.movie_id = id
		self.url = "http://film.wp.pl/id,%s,film_szczegoly.html" % self.movie_id
		self.encode='iso-8859-2'

	def initialize(self):
		self.page = gutils.trim(self.page, '<h1 class="mp0">', '</div>\n</div>')
		self.cast_page = self.open_page(url="http://film.wp.pl/id,%s,film_obsada_i_tworcy.html" % self.movie_id)
		self.cast_page = gutils.trim(self.cast_page, '<h1 class="mp0">', "</div>\r\n</div>\r\n")

	def get_image(self):
		self.image_url = gutils.trim(self.page, '<img src="', '" name="o')

	def get_o_title(self):
		self.o_title = gutils.trim(self.page, '<b>Tytu\xB3 orygina\xB3u:</b>', "\t\t</div><div")
		self.o_title = gutils.clean(self.o_title)
		if self.o_title == '':
			self.o_title = gutils.trim(self.page, '</h1>', '<div')

	def get_title(self):
		self.title = gutils.trim(self.page, '<b>Tytu\xB3 polski:</b>', "\t\t</div><div")
		if self.title == '':
			self.title = gutils.before(self.page, '</h1>')
			pos = string.find(self.title, '(')
			if pos > -1:
				self.title = self.title[:pos]

	def get_director(self):
		self.director = gutils.trim(self.cast_page, '>re\xBFyser\t</div>', '</div>')

	def get_plot(self):
		self.plot = gutils.trim(self.page, ' />', '\t\t<div class="clr">')
		pos = string.find(self.plot, "<br>Zobacz tak\xBFe:<br>")
		if pos > 0:
			self.plot = self.plot[:pos]
		self.plot = string.replace(self.plot,"\r\n\r\n", '')

	def get_year(self):
		self.year = gutils.trim(self.page, '<b>Rok produkcji:</b>', "\t\t</div><div")
		year = re.findall(r'\d+', self.year)
		if len(year)>0:
			self.year = year[0]

	def get_runtime(self):
		self.runtime = gutils.trim(self.page, '<b>Czas trwania:</b>', 'min')
		runtime =  re.findall(r'\d+', self.runtime)
		if len(runtime)>0:
			self.runtime = runtime

	def get_genre(self):
		self.genre = gutils.trim(self.page, '<b>Gatunek:</b>', "\t\t</div><div")
		self.genre = string.replace(self.genre, '<br />	', '')
		self.genre = string.replace(self.genre, '<br />', ' / ')

	def get_cast(self):
		self.cast = gutils.trim(self.cast_page, '<h2>OBSADA:</h2>', '<div class="b')
		self.cast = gutils.after(self.cast, '<div class="clr"></div>')
		self.cast = string.replace(self.cast, '\t', '')
		self.cast = string.replace(self.cast, '\n</div>\n<', _(' as ')+'<')
		self.cast = gutils.strip_tags(self.cast)
		self.cast = string.replace(self.cast,  "%s\n" % _(' as '), "\n")

	def get_classification(self):
		self.classification = gutils.trim(self.page, '<b>Przedzia\xB3 wiekowy:</b>', "\t\t</div><div")
		self.classification = gutils.trim(self.classification, 'od ', ' ')

	def get_studio(self):
		self.studio = gutils.trim(self.page, '<b>Wytw\xF3rnia:</b>', "\t\t</div><div")

	def get_o_site(self):
		self.o_site = ''

	def get_site(self):
		self.site = "http://film.wp.pl/id,%s,film.html" % self.movie_id

	def get_trailer(self):
		self.trailer = "http://film.wp.pl/id,%s,film_trailer.html" % self.movie_id

	def get_country(self):
		pos = self.page.find('<b>Kraj produkcji:</b>')
		if pos == -1:
			pos = self.page.find('<b>Kraje produkcji:</b>')
		if pos > -1:
			self.country = gutils.trim(self.page[pos:], '</b>', "\t\t</div><div")

	def get_rating(self):
		self.rating = None

	def get_notes(self):
		self.notes = ''

class SearchPlugin(movie.SearchMovie):
	def __init__(self):
		self.encode='iso-8859-2'
		self.original_url_search	= 'http://film.wp.pl/szukaj,%s,type,f,szukaj.html'
		self.translated_url_search	= 'http://film.wp.pl/szukaj,%s,type,f,szukaj.html'

	def search(self,parent_window):
		self.open_search(parent_window)
		self.page = gutils.trim(self.page, '<div id="filmUS"', '<div id=');
		return self.page

	def get_searches(self):
		elements = string.split(self.page, '</div><br />')
		self.number_results = elements[-1]

		if elements[0] != '':
			for element in elements:
				tmp_id = gutils.trim(element,'<div class="rgt"><a href="id,', ',')
				self.ids.append(tmp_id)
				tmp_title = gutils.trim(element, 'html"><b>', '</b>')
				self.titles.append(tmp_title)
		else:
			self.number_results = 0



# -*- coding: UTF-8 -*-

__revision__ = '$Id: gutils.py 1040 2008-11-15 21:13:49Z mikej06 $'

# Copyright (c) 2005-2008 Vasco Nunes, Piotr Ożarowski
#
# This program is free software; you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation; either version 2 of the License, or
# (at your option) any later version.
#
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU Library General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with this program; if not, write to the Free Software
# 51 Franklin Street, Fifth Floor, Boston, MA  02110-1301, USA

# You may use and distribute this software under the terms of the
# GNU General Public License, version 2 or later

import string
import os
try:
	import gtk
	import gobject
except:
	pass
import htmlentitydefs
import re
import webbrowser

url_re = re.compile('^\w+://')
entity = re.compile(r'\&.\w*?\;')
html_tags = re.compile(r'\<.*?\>')

def remove_accents(txt, encoding='iso-8859-1'):
	d = {192: u'A', 193: u'A', 194: u'A', 195: u'A', 196: u'A', 197: u'A',
			199: u'C', 200: u'E', 201: u'E', 202: u'E', 203: u'E', 204: u'I',
			205: u'I', 206: u'I', 207: u'I', 209: u'N', 210: u'O', 211: u'O',
			212: u'O', 213: u'O', 214: u'O', 216: u'O', 217: u'U', 218: u'U',
			219: u'U', 220: u'U', 221: u'Y', 224: u'a', 225: u'a', 226: u'a',
			227: u'a', 228: u'a', 229: u'a', 231: u'c', 232: u'e', 233: u'e',
			234: u'e', 235: u'e', 236: u'i', 237: u'i', 238: u'i', 239: u'i',
			241: u'n', 242: u'o', 243: u'o', 244: u'o', 245: u'o', 246: u'o',
			248: u'o', 249: u'u', 250: u'u', 251: u'u', 252: u'u', 253: u'y',
			255: u'y'}
	return unicode(txt, encoding).translate(d)

def is_number(x):
	return isinstance(x, int)

def find_next_available(db):
	"""
	finds next available movie number.
	This is the first empty position.
	If none is empty then increments the last position.
	"""
	first = 0

	movies = db.Movie.select(order_by="number ASC")
	for movie in movies:
		second = int(movie.number)
		if second is None:
			second = 0
		if (second>first+1):
			break
		first = second

	if first is None:
		return 1
	else:
		number = first+1
		return number

def trim(text,key1,key2):
	p1 = string.find(text,key1)
	if p1 == -1:
		return ""
	else:
		p1 = p1+len(key1)
	p2 = string.find(text[p1:],key2)
	if p2 == -1:
		return ""
	else:
		p2 = p1+p2
	return text[p1:p2]

def after(text,key):
	p1 = string.find(text,key)
	return text[p1+len(key):]

def before(text,key):
	p1 = string.find(text,key)
	return text[:p1]

def gescape(text):
	text=string.replace(text,"'", "''")
	text=string.replace(text,"--", "-")
	return text

def progress(blocks,size_block,size):
	transfered = blocks * size_block
	if size > 0 and transfered > size:
		transfered = size
	elif size < 0:
		size = "?"
	print transfered, '/', size, 'bytes'

# functions to handle comboboxentry stuff

def set_model_from_list (cb, items):
	"""Setup a ComboBox or ComboBoxEntry based on a list of strings."""
	model = gtk.ListStore(str)
	for i in items:
		model.append([i])
	cb.set_model(model)
	if type(cb) == gtk.ComboBoxEntry:
		cb.set_text_column(0)
	elif type(cb) == gtk.ComboBox:
		cell = gtk.CellRendererText()
		cb.pack_start(cell, True)
		cb.add_attribute(cell, 'text', 0)

def on_combo_box_entry_changed(widget):
	model = widget.get_model()
	m_iter = widget.get_active_iter()
	if m_iter:
		return model.get_value(m_iter, 0)
	else:
		return 0

def on_combo_box_entry_changed_name(widget):
	return widget.get_active_text().decode('utf-8')

def convert_entities(text):
	def conv(ents):
		entities = htmlentitydefs.entitydefs
		ents = ents.group(0)
		ent_code = entities.get(ents[1:-1], None)
		if ent_code:
			try:
				ents = unicode(ent_code, 'UTF-8')
			except UnicodeDecodeError:
				ents = unicode(ent_code, 'latin-1')
			except Exception, ex:
				print("error occurred while converting entity %s: %s" % (ents, ex))

			# check if it still needs conversion
			if not entity.search(ents):
				return ents

		if ents[1] == '#':
			code = ents[2:-1]
			base = 10
			if code[0] == 'x':
				code = code[1:]
				base = 16
			return unichr(int(code, base))
		else:
			return

	in_entity = entity.search(text)
	if not in_entity:
		return text
	else:
		ctext = in_entity.re.sub(conv, text)
		return ctext

def strip_tags(text):
	if text is None:
		return ''
	finished = 0
	while not finished:
		finished = 1
		# check if there is an open tag left
		start = text.find("<")
		if start >= 0:
			# if there is, check if the tag gets closed
			stop = text[start:].find(">")
			if stop >= 0:
				# if it does, strip it, and continue loop
				text = text[:start] + text[start+stop+1:]
				finished = 0
	return text

def save_pixmap(self, pixmap, filename):
	pixmap.save(filename, "jpeg", {"quality":"70"})

def clean(text):
	t = strip_tags(text)
	t = string.replace(t, '&nbsp;', ' ')
	t = string.replace(t, '&#34;', '')
	t = string.replace(t, '&#160;', ' ')
	return string.strip(t)

def gdecode(txt, encode):
	try:
		return txt.decode(encode)
	except:
		return txt

# Messages

def error(self, msg, parent=None):
	dialog = gtk.MessageDialog(parent,
			gtk.DIALOG_MODAL | gtk.DIALOG_DESTROY_WITH_PARENT,
			gtk.MESSAGE_ERROR, gtk.BUTTONS_OK, msg)
	dialog.set_skip_taskbar_hint(False)
	dialog.run()
	dialog.destroy()

def urllib_error(msg, parent=None):
	dialog = gtk.MessageDialog(parent,
			gtk.DIALOG_MODAL | gtk.DIALOG_DESTROY_WITH_PARENT,
			gtk.MESSAGE_ERROR, gtk.BUTTONS_OK, msg)
	dialog.set_skip_taskbar_hint(False)
	dialog.run()
	dialog.destroy()

def warning(self, msg, parent=None):
	dialog = gtk.MessageDialog(parent,
			gtk.DIALOG_MODAL | gtk.DIALOG_DESTROY_WITH_PARENT,
			gtk.MESSAGE_WARNING, gtk.BUTTONS_OK, msg)
	dialog.set_skip_taskbar_hint(False)
	dialog.run()
	dialog.destroy()

def info(self, msg, parent=None):
	dialog = gtk.MessageDialog(parent,
			gtk.DIALOG_MODAL | gtk.DIALOG_DESTROY_WITH_PARENT,
			gtk.MESSAGE_INFO, gtk.BUTTONS_OK, msg)
	dialog.set_skip_taskbar_hint(False)
	dialog.run()
	dialog.destroy()

def question(self, msg, cancel=1, parent=None):
	if not parent: parent = self
	dialog = gtk.MessageDialog(parent,
			gtk.DIALOG_MODAL | gtk.DIALOG_DESTROY_WITH_PARENT,
			gtk.MESSAGE_QUESTION, gtk.BUTTONS_NONE, msg)
	dialog.add_buttons(gtk.STOCK_YES, gtk.RESPONSE_YES,
			gtk.STOCK_NO, gtk.RESPONSE_NO)
	dialog.set_default_response(gtk.RESPONSE_NO)
	dialog.set_skip_taskbar_hint(False)
	response = dialog.run()
	dialog.destroy()
	return response

def file_chooser(title, action=None, buttons=None, name="", folder=os.path.expanduser("~"), picture = False):
	dialog = gtk.FileChooserDialog(title=title, action=action, buttons=buttons)
	dialog.set_default_response(gtk.RESPONSE_OK)
	if name:
		dialog.set_current_name(name)
	if folder:
		dialog.set_current_folder(folder)
	mfilter = gtk.FileFilter()
	if picture==True:
		preview = gtk.Image()
		dialog.set_preview_widget(preview)
		dialog.connect("update-preview", update_preview_cb, preview)
		mfilter.set_name(_("Images"))
		mfilter.add_mime_type("image/png")
		mfilter.add_mime_type("image/jpeg")
		mfilter.add_mime_type("image/gif")
		mfilter.add_pattern("*.[pP][nN][gG]")
		mfilter.add_pattern("*.[jJ][pP][eE][gG]")
		mfilter.add_pattern("*.[gG][iI][fF]")
		mfilter.add_pattern("*.[tT][iI][fF]{1,2}")
		mfilter.add_pattern("*.[xX][pP][mM]")
		dialog.add_filter(mfilter)
	mfilter = gtk.FileFilter()
	mfilter.set_name(_("All files"))
	mfilter.add_pattern("*")
	dialog.add_filter(mfilter)


	response = dialog.run()
	if response == gtk.RESPONSE_OK:
		filename = dialog.get_filename()
	elif response == gtk.RESPONSE_CANCEL:
		filename = None
	else:
		return False
	path = dialog.get_current_folder()
	dialog.destroy()
	return filename, path

def update_preview_cb(file_chooser, preview):
	filename = file_chooser.get_preview_filename()
	try:
		pixbuf = gtk.gdk.pixbuf_new_from_file_at_size(filename, 128, 128)
		preview.set_from_pixbuf(pixbuf)
		have_preview = True
	except:
		have_preview = False
	file_chooser.set_preview_widget_active(have_preview)
	return

def run_browser(url):
	webbrowser.register('open', webbrowser.GenericBrowser("open '%s'"))
	webbrowser._tryorder.append('open')
	webbrowser.open(url)

def read_plugins(prefix,directory):
	"""returns available plugins"""
	import glob
	return glob.glob("%s/%s*.py" % (directory,prefix) )

def findKey(val, dict):
	for key, value in dict.items():
		if value == val: return key
	return None

def garbage(handler):
	pass

def make_thumbnail(self, file_name):
	source = os.path.join(self.locations['posters'], file_name)
	if os.path.isfile(source):
		self.Image.set_from_file(source)
		pixbuf = self.Image.get_pixbuf()
		pixbuf = pixbuf.scale_simple(30, 40, 'bilinear')
		save_pixmap(self, pixbuf, os.path.join(self.locations['posters'], "t_%s"%file_name))
	else:
		return 0

def make_medium_image(self, file_name):
	source = os.path.join(self.locations['posters'], file_name)
	if os.path.isfile(source):
		self.Image.set_from_file(source)
		pixbuf = self.Image.get_pixbuf()
		pixbuf = pixbuf.scale_simple(100, 140, 'bilinear')
		save_pixmap(self, pixbuf, os.path.join(self.locations['posters'], "m_%s"%file_name))
	else:
		return 0

def clean_posters_dir(self):
	posters_dir = self.locations['posters']

	counter = 0

	for files in os.walk(posters_dir):
		for names in files:
			for name in names:
				if name.startswith('poster'):
					# lets check if this poster is orphan
					used = self.db.Movie.count_by(image=string.replace(name,".jpg",""))
					if not used:
						counter += 1
						os.unlink(os.path.join(posters_dir, name))
						m_file = os.path.join(posters_dir, "m_"+name)
						if os.path.isfile(m_file):
							os.unlink(m_file)
						t_file = os.path.join(posters_dir, "t_"+name)
						if os.path.isfile(t_file):
							os.unlink(t_file)

	if counter:
		print "%d orphan files cleaned."%counter
	else:
		print "No orphan files found."

def decompress(data):
	import gzip, StringIO
	try:
		compressedStream = StringIO.StringIO(data)
		gzipper = gzip.GzipFile(fileobj=compressedStream)
		data = gzipper.read()
	except:
		pass
	return data

def get_dependencies():
	depend = []
	try:
		import gtk
		version	= '.'.join([str(i) for i in gtk.pygtk_version])
		if gtk.pygtk_version <= (2, 6, 0):
			version = '-%s' % version
	except:
		version = False
	depend.append({'module': 'gtk',
		'version'	: version,
		'module_req'	: '2.6',
		'url'		: 'http://www.pygtk.org/',
		'debian'	: 'python-gtk2',
		'debian_req'	: '2.8.6-1'
		# TODO: 'fedora', 'suse', etc.
	})
	try:
		import gtk.glade
		# (version == gtk.pygtk_version)
	except:
		version = False
	depend.append({'module': 'gtk.glade',
		'version'	: version,
		'module_req'	: '2.6',
		'url'		: 'http://www.pygtk.org/',
		'debian'	: 'python-glade2',
		'debian_req'	: '2.8.6-1'
	})
	try:
		import sqlalchemy
		version = True
	except:
		version = False
	depend.append({'module': 'sqlalchemy',
		'version'	: version,
		'module_req'	: '0.3.10',
		'url'		: 'http://www.sqlalchemy.org/',
		'debian'	: 'python-sqlalchemy',
		'debian_req'	: '0.3.10-1'
	})
	try:
		import sqlite3
		version = sqlite3.version
	except ImportError:
		version = False
	if version is False:
		try:
			import pysqlite2
			version = True
		except:
			version = False
		depend.append({'module': 'pysqlite2',
			'version'	: version,
			'url'		: 'http://initd.org/tracker/pysqlite',
			'debian'	: 'python-pysqlite2',
			'debian_req'	: '2.3.0-1'
		})
	else:
		depend.append({'module': 'sqlite3',
			'version'	: version,
			'url'		: 'http://www.python.org',
			'debian'	: 'python',
			'debian_req'	: '2.5'
		})
	try:
		import reportlab
		version = reportlab.Version
	except:
		version = False
	depend.append({'module': 'reportlab',
		'version'	: version,
		'url'		: 'http://www.reportlab.org/',
		'debian'	: 'python-reportlab',
		'debian_req'	: '1.20debian-6'
	})
	try:
		import PIL
		version = True
	except:
		version = False
	depend.append({'module': 'PIL',
		'version'	: version,
		'url'		: 'http://www.pythonware.com/products/pil/',
		'debian'	: 'python-imaging',
		'debian_req'	: '1.1.5-6'
	})
	try:
		import xml
		version = xml.__version__
	except:
		version = False
	depend.append({'module': 'xml',
		'version'	: version,
		'url'		: 'http://pyxml.sf.net/',
		'debian'	: 'python-xml'
	})
	# extra dependencies:
	optional = []
	try:
		import psycopg2
		version = psycopg2.__version__
	except:
		version = False
	optional.append({'module': 'psycopg2',
		'version'	: version,
		'url'		: 'http://initd.org/tracker/psycopg/wiki/PsycopgTwo',
		'debian'	: 'python-psycopg2',
		'debian_req'	: '1.1.21-6'
	})
	try:
		import MySQLdb
		version	= '.'.join([str(i) for i in MySQLdb.version_info])
	except:
		version = False
	optional.append({'module': 'MySQLdb',
		'version'	: version,
		'url'		: 'http://sourceforge.net/projects/mysql-python',
		'debian'	: 'python-mysqldb',
		'debian_req'	: '1.2.1-p2-2'
	})
	try:
		import chardet
		version	= chardet.__version__
	except:
		version = False
	optional.append({'module': 'chardet',
		'version'	: version,
		'url'		: 'http://chardet.feedparser.org/',
		'debian'	: 'python-chardet'
	})
	try:
		import sqlite
		version	= sqlite.version
	except:
		version = False
	optional.append({'module': 'sqlite',
		'version'	: version,
		'url'		: 'http://initd.org/tracker/pysqlite',
		'debian'	: 'python-sqlite'
	})
	return depend, optional


def html_encode(s):
	if not isinstance(s, str):
		s = str(s)
	s = s.replace('&', '&amp;')
	s = s.replace('<', '&lt;')
	s = s.replace('>', '&gt;')
	s = s.replace('"', '&quot;')
	return s

def digits_only(s, maximum=None):
	if s is None:
		return 0
	import string, re
	match = re.compile(r"\d+")
	a = match.findall(str(s))
	if len(a):
		s = int(a[0])
	else:
		s = 0
	if maximum is None:
		return s
	else:
		if s > maximum:
			return maximum
		else:
			return s

def copytree(src, dst, symlinks=False):
	"""Recursively copy a directory tree using copy2().

	This is shutil's copytree modified version
	"""
	from shutil import copy2
	names = os.listdir(src)
	if not os.path.isdir(dst):
		os.mkdir(dst)
	errors = []
	for name in names:
		srcname = os.path.join(src, name)
		dstname = os.path.join(dst, name)
		try:
			if symlinks and os.path.islink(srcname):
				linkto = os.readlink(srcname)
				os.symlink(linkto, dstname)
			elif os.path.isdir(srcname):
				copytree(srcname, dstname, symlinks)
			else:
				copy2(srcname, dstname)
		except (IOError, os.error), why:
			errors.append((srcname, dstname, why))
		# catch the Error from the recursive copytree so that we can
		# continue with other files
		except EnvironmentError, err:
			errors.extend(err.args[0])
	if errors:
		raise EnvironmentError, errors

def is_windows_system():
	if os.name == 'nt' or os.name.startswith('win'): # win32, win64
		return True
	return False
